/**
 * @file Iwlbitops.c
 *
 * @details
 * This header file contains the code to implement bit wise
 * operation for Intel UNDI driver.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2017 Dell Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * BSD LICENSE
 *
 * Copyright(c) 2007 Dell Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Dell Inc nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include "IwlBitOps.h"

#define BITOP_WORD(nr)      ((nr) / BITS_PER_LONG)

unsigned long __ffs(
    unsigned long word
    )
{
    int num = 0;

#if BITS_PER_LONG == 64
    if ((word & 0xffffffff) == 0) {
        num += 32;
        word >>= 32;
    }
#endif
    if ((word & 0xffff) == 0) {
        num += 16;
        word >>= 16;
    }
    if ((word & 0xff) == 0) {
        num += 8;
        word >>= 8;
    }
    if ((word & 0xf) == 0) {
        num += 4;
        word >>= 4;
    }
    if ((word & 0x3) == 0) {
        num += 2;
        word >>= 2;
    }
    if ((word & 0x1) == 0)
        num += 1;
    return num;
}

unsigned long __fls(
    unsigned long word
    )
{
    int num = BITS_PER_LONG - 1;
    unsigned long test = ~0ul;

    test <<= 16;

 #if BITS_PER_LONG == 64
    if (!(word & (test << 16))) {

        num -= 32;

        word <<= 32;

    }
 #endif


    if (!(word & (~0ul << (BITS_PER_LONG-16)))) {
        num -= 16;

        word <<= 16;
    }

    if (!(word & (~0ul << (BITS_PER_LONG-8)))) {
        num -= 8;

        word <<= 8;
    }
    if (!(word & (~0ul << (BITS_PER_LONG-4)))) {
        num -= 4;

        word <<= 4;
    }
    if (!(word & (~0ul << (BITS_PER_LONG-2)))) {
        num -= 2;

        word <<= 2;
    }

    if (!(word & (~0ul << (BITS_PER_LONG-1))))
        num -= 1;

    return num;
}




/*
 * Find the first cleared bit in a memory region.
 */
unsigned long find_first_zero_bit(
    const unsigned long *addr,
    unsigned long size
    )
{
    const unsigned long *p = addr;
    unsigned long result = 0;
    unsigned long tmp=0;

    while (size & ~(BITS_PER_LONG-1)) {
        if (~(tmp = *(p++)))
            goto found;
        result += BITS_PER_LONG;
        size -= BITS_PER_LONG;
    }
    if (!size)
        return result;

    tmp = (*p) | (~0UL << size);
    if (tmp == ~0UL)    /* Are any bits zero? */
        return result + size;   /* Nope. */
found:
    return result + __ffs(~tmp);
}

unsigned long find_next_bit(
    const unsigned long *addr,
    unsigned long size,
    unsigned long offset
    )
{
    const unsigned long *p = (addr + BITOP_WORD(offset));
    unsigned long result = offset & ~(BITS_PER_LONG-1);
    unsigned long tmp=0;

    if (offset >= size)
        return size;
    size -= result;
    offset %= BITS_PER_LONG;
    if (offset) {
        tmp = *(p++);
        tmp &= (~0UL << offset);
        if (size < BITS_PER_LONG)
            goto found_first;
        if (tmp)
            goto found_middle;
        size -= BITS_PER_LONG;
        result += BITS_PER_LONG;
    }
    while (size & ~(BITS_PER_LONG-1)) {
        tmp = *(p++);

        if (tmp)
            goto found_middle;
        result += BITS_PER_LONG;
        size -= BITS_PER_LONG;
    }
    if (!size)
        return result;
    tmp = *p;

found_first:
    tmp &= (~0UL >> (BITS_PER_LONG - size));
    if (tmp == 0UL)     /* Are any bits set? */
        return result + size;   /* Nope. */
found_middle:
    return result + __ffs(tmp);
}

unsigned long find_first_bit(
    const unsigned long *addr,
    unsigned long size
    )
{
    const unsigned long *p = addr;
    unsigned long result = 0;
    unsigned long tmp=0;

    while (size & ~(BITS_PER_LONG-1)) {
        tmp = *(p++);
//		if ((tmp = *(p++)))
        if (tmp)
            goto found;
        result += BITS_PER_LONG;
        size -= BITS_PER_LONG;
    }
    if (!size)
        return result;

    tmp = (*p) & (~0UL >> (BITS_PER_LONG - size));
    if (tmp == 0UL)     /* Are any bits set? */
        return result + size;   /* Nope. */
found:
    return result + __ffs(tmp);
}


unsigned long find_last_bit(
    const unsigned long *addr,
    unsigned long size
    )
{
    unsigned long words=0;
    unsigned long tmp=0;

    /* Start at final word. */
    words = size / BITS_PER_LONG;

    /* Partial final word? */
    if (size & (BITS_PER_LONG-1)) {
        tmp = (addr[words] & (~0UL >> (BITS_PER_LONG
                                       - (size & (BITS_PER_LONG-1)))));
        if (tmp)
            goto found;
    }

    while (words) {
        tmp = addr[--words];
        if (tmp) {
found:
            return words * BITS_PER_LONG + __fls(tmp);
        }
    }

    /* Not found */
    return size;
}

#define for_each_set_bit(bit, addr, size) \
    for ((bit) = find_first_bit((addr), (size));        \
         (bit) < (size);                    \
         (bit) = find_next_bit((addr), (size), (bit) + 1))

