/**
 * @file IwlErrno.h
 *
 * @details
 * This header file contains error code macros for Intel UNDI
 * driver.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2017 Dell Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * BSD LICENSE
 *
 * Copyright(c) 2007 Dell Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Dell Inc nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef ERRNO_H_
#define ERRNO_H_

#define EPERM        1  /* Operation not permitted */
#define ENOENT       2  /* No such file or directory */
#define ESRCH        3  /* No such process */
#define EINTR        4  /* Interrupted system call */
#define EIO      5  /* I/O error */
#define ENXIO        6  /* No such device or address */
#define E2BIG        7  /* Argument list too long */
#define ENOEXEC      8  /* Exec format error */
#define EBADF        9  /* Bad file number */
#define ECHILD      10  /* No child processes */
#define EAGAIN      11  /* Try again */
#define ENOMEM      12  /* Out of memory */
#define EACCES      13  /* Permission denied */
#define EFAULT      14  /* Bad address */
#define ENOTBLK     15  /* Block device required */
#define EBUSY       16  /* Device or resource busy */
#define EEXIST      17  /* File exists */
#define EXDEV       18  /* Cross-device link */
#define ENODEV      19  /* No such device */
#define ENOTDIR     20  /* Not a directory */
#define EISDIR      21  /* Is a directory */
#define EINVAL      22  /* Invalid argument */
#define ENFILE      23  /* File table overflow */
#define EMFILE      24  /* Too many open files */
#define ENOTTY      25  /* Not a typewriter */
#define ETXTBSY     26  /* Text file busy */
#define EFBIG       27  /* File too large */
#define ENOSPC      28  /* No space left on device */
#define ESPIPE      29  /* Illegal seek */
#define EROFS       30  /* Read-only file system */
#define EMLINK      31  /* Too many links */
#define EPIPE       32  /* Broken pipe */
#define EDOM        33  /* Math argument out of domain of func */
#define ERANGE      34  /* Math result not representable */

/* These match the SunOS error numbering scheme. */

#define EWOULDBLOCK EAGAIN  /* Operation would block */
#define EINPROGRESS 36  /* Operation now in progress */
#define EALREADY    37  /* Operation already in progress */
#define ENOTSOCK    38  /* Socket operation on non-socket */
#define EDESTADDRREQ    39  /* Destination address required */
#define EMSGSIZE    40  /* Message too long */
#define EPROTOTYPE  41  /* Protocol wrong type for socket */
#define ENOPROTOOPT 42  /* Protocol not available */
#define EPROTONOSUPPORT 43  /* Protocol not supported */
#define ESOCKTNOSUPPORT 44  /* Socket type not supported */
#define EOPNOTSUPP  45  /* Op not supported on transport endpoint */
#define EPFNOSUPPORT    46  /* Protocol family not supported */
#define EAFNOSUPPORT    47  /* Address family not supported by protocol */
#define EADDRINUSE  48  /* Address already in use */
#define EADDRNOTAVAIL   49  /* Cannot assign requested address */
#define ENETDOWN    50  /* Network is down */
#define ENETUNREACH 51  /* Network is unreachable */
#define ENETRESET   52  /* Net dropped connection because of reset */
#define ECONNABORTED    53  /* Software caused connection abort */
#define ECONNRESET  54  /* Connection reset by peer */
#define ENOBUFS     55  /* No buffer space available */
#define EISCONN     56  /* Transport endpoint is already connected */
#define ENOTCONN    57  /* Transport endpoint is not connected */
#define ESHUTDOWN   58  /* No send after transport endpoint shutdown */
#define ETOOMANYREFS    59  /* Too many references: cannot splice */
#define ETIMEDOUT   60  /* Connection timed out */
#define ECONNREFUSED    61  /* Connection refused */
#define ELOOP       62  /* Too many symbolic links encountered */
#define ENAMETOOLONG    63  /* File name too long */
#define EHOSTDOWN   64  /* Host is down */
#define EHOSTUNREACH    65  /* No route to host */
#define ENOTEMPTY   66  /* Directory not empty */
#define EPROCLIM        67      /* SUNOS: Too many processes */
#define EUSERS      68  /* Too many users */
#define EDQUOT      69  /* Quota exceeded */
#define ESTALE      70  /* Stale file handle */
#define EREMOTE     71  /* Object is remote */
#define ENOSTR      72  /* Device not a stream */
#define ETIME       73  /* Timer expired */
#define ENOSR       74  /* Out of streams resources */
#define ENOMSG      75  /* No message of desired type */
#define EBADMSG     76  /* Not a data message */
#define EIDRM       77  /* Identifier removed */
#define EDEADLK     78  /* Resource deadlock would occur */
#define ENOLCK      79  /* No record locks available */
#define ENONET      80  /* Machine is not on the network */
#define ERREMOTE        81      /* SunOS: Too many lvls of remote in path */
#define ENOLINK     82  /* Link has been severed */
#define EADV        83  /* Advertise error */
#define ESRMNT      84  /* Srmount error */
#define ECOMM       85      /* Communication error on send */
#define EPROTO      86  /* Protocol error */
#define EMULTIHOP   87  /* Multihop attempted */
#define EDOTDOT     88  /* RFS specific error */
#define EREMCHG     89  /* Remote address changed */
#define ENOSYS      90  /* Function not implemented */

/* The rest have no SunOS equivalent. */
#define ESTRPIPE    91  /* Streams pipe error */
#define EOVERFLOW   92  /* Value too large for defined data type */
#define EBADFD      93  /* File descriptor in bad state */
#define ECHRNG      94  /* Channel number out of range */
#define EL2NSYNC    95  /* Level 2 not synchronized */
#define EL3HLT      96  /* Level 3 halted */
#define EL3RST      97  /* Level 3 reset */
#define ELNRNG      98  /* Link number out of range */
#define EUNATCH     99  /* Protocol driver not attached */
#define ENOCSI      100 /* No CSI structure available */
#define EL2HLT      101 /* Level 2 halted */
#define EBADE       102 /* Invalid exchange */
#define EBADR       103 /* Invalid request descriptor */
#define EXFULL      104 /* Exchange full */
#define ENOANO      105 /* No anode */
#define EBADRQC     106 /* Invalid request code */
#define EBADSLT     107 /* Invalid slot */
#define EDEADLOCK   108 /* File locking deadlock error */
#define EBFONT      109 /* Bad font file format */
#define ELIBEXEC    110 /* Cannot exec a shared library directly */
#define ENODATA     111 /* No data available */
#define ELIBBAD     112 /* Accessing a corrupted shared library */
#define ENOPKG      113 /* Package not installed */
#define ELIBACC     114 /* Can not access a needed shared library */
#define ENOTUNIQ    115 /* Name not unique on network */
#define ERESTART    116 /* Interrupted syscall should be restarted */
#define EUCLEAN     117 /* Structure needs cleaning */
#define ENOTNAM     118 /* Not a XENIX named type file */
#define ENAVAIL     119 /* No XENIX semaphores available */
#define EISNAM      120 /* Is a named type file */
#define EREMOTEIO   121 /* Remote I/O error */
#define EILSEQ      122 /* Illegal byte sequence */
#define ELIBMAX     123 /* Atmpt to link in too many shared libs */
#define ELIBSCN     124 /* .lib section in a.out corrupted */

#define ENOMEDIUM   125 /* No medium found */
#define EMEDIUMTYPE 126 /* Wrong medium type */
#define ECANCELED   127 /* Operation Cancelled */
#define ENOKEY      128 /* Required key not available */
#define EKEYEXPIRED 129 /* Key has expired */
#define EKEYREVOKED 130 /* Key has been revoked */
#define EKEYREJECTED    131 /* Key was rejected by service */

/* for robust mutexes */
#define EOWNERDEAD  132 /* Owner died */
#define ENOTRECOVERABLE 133 /* State not recoverable */

#define ERFKILL     134 /* Operation not possible due to RF-kill */

#define EHWPOISON   135 /* Memory page has hardware error */

#define EINVLN      136

#define EDEVERROR   137  /* NIC ERROR*/

#endif /* ERRNO_H_ */
