/**
 * @file IwlMvmOps.c
 *
 * @details
 * This file contains code to implement operations on NIC for
 * UNDI driver.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2017 Dell Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * BSD LICENSE
 *
 * Copyright(c) 2007 Dell Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Dell Inc nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include <Uefi.h>
#include <Library/DebugLib.h>
#include <Library/BaseLib.h>
#include <Library/UefiLib.h>
#include <Library/BaseMemoryLib.h>
#include <Library/UefiRuntimeServicesTableLib.h>
#include <Library/UefiBootServicesTableLib.h>
#include "IwlMvmOps.h"

STATIC IWL_OP_MODE *OpMode = NULL;

IWL_OP_MODE* GetMvmOpsProtocol()
{
    EFI_STATUS Status = EFI_SUCCESS;
    IWL_OP_MODE_OPS *OpModeOps = NULL;

    Status = (EFI_STATUS)gBS->LocateProtocol(
            &gIntelMvmOpsProtocolGuid,
            NULL,
            (VOID **)&OpModeOps
            );
    if(EFI_ERROR(Status))
    {
        DEBUG( (EFI_D_ERROR, "%s:%d:%s(): Failed to locate the mvm ops protocol: 0x%x\n", __FILE__, __LINE__, __FUNCTION__, Status ));
        OpMode = NULL;
    }
    else
    {
        OpMode = OpModeOps->getContext();
        if(OpMode == NULL)
        {
            DEBUG( (EFI_D_ERROR, "%s:%d:%s(): Failed to get OpMode context.\n", __FILE__, __LINE__, __FUNCTION__ ));
        }
    }

    return OpMode;
}

VOID
Iwl_Mvm_Nic_Error_Stub()
{
    if(OpMode == NULL)
    {
        OpMode = GetMvmOpsProtocol();
    }

    if(OpMode != NULL)
        OpMode->Ops->nic_error(OpMode);
}

INT32
Iwl_Mvm_Rx_Dispatch_Stub(
        struct iwl_rx_cmd_buffer *rxb,
        struct iwl_device_cmd *cmd,
        BOOLEAN page_stolen)
{
    if(OpMode == NULL)
    {
        OpMode = GetMvmOpsProtocol();
    }

    if(OpMode != NULL)
    {
        return OpMode->Ops->rx(OpMode, rxb, cmd, page_stolen);
    }
    else
    {
        return 0;
    }
}

VOID
iwl_op_mode_cmd_queue_full_Stub()
{
    if(OpMode == NULL)
    {
        OpMode = GetMvmOpsProtocol();
    }

    if(OpMode != NULL)
    {
           OpMode->Ops->cmd_queue_full(OpMode);
    }
}

VOID
iwl_op_mode_queue_full_Stub(
    IN INT32 queue
    )
{
    if(OpMode == NULL)
    {
        OpMode = GetMvmOpsProtocol();
    }

    if(OpMode != NULL)
    {
        OpMode->Ops->queue_full(OpMode, queue);
    }
}

BOOLEAN
Iwl_Mvm_Set_Hw_Rfkill_State_Stub(BOOLEAN State)
{
    if(OpMode == NULL)
    {
        OpMode = GetMvmOpsProtocol();
    }

    if(OpMode != NULL)
    {
       return OpMode->Ops->hw_rf_kill(OpMode, State);
    }
    else
    {
        return 0;
    }
}

VOID
Iwl_Mvm_Nic_Config_Stub()
{
    if(OpMode == NULL)
    {
        OpMode = GetMvmOpsProtocol();
    }

    if(OpMode != NULL)
    {
        OpMode->Ops->nic_config(OpMode);
    }
}

VOID Lq_Cmd_Enqueue_Stub(struct iwl_lq_cmd *lq, INTN index)
{
    if(OpMode == NULL)
    {
        OpMode = GetMvmOpsProtocol();
    }

    if(OpMode != NULL)
    {
        OpMode->Ops->Lq_Cmd_Enqueue(lq, index);
    }
}

