/**
 * @file IwlTransMain.c
 *
 * @details
 * This file contains code to define and install the Iwl Trans protocol.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2017 Dell Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * BSD LICENSE
 *
 * Copyright(c) 2007 Dell Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Dell Inc nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include "iwl-trans-context.h"
#include "IwlTransMain.h"
#include "internal.h"
#include "rs.h"
#include "fw.h"
#include <Uefi.h>

INTEL_TRANS_OPS_PROTOCOL PcieTransOps = {
    Iwl_Trans_Pcie_Initialize,
    Iwl_Trans_Pcie_Free,
	Iwl_Trans_Pcie_Start_Hw,
    Iwl_Trans_Pcie_Start_Fw,
    Iwl_Trans_Pcie_Fw_Alive,
    iwl_trans_pcie_send_hcmd,
    iwl_trans_pcie_tx,
    iwl_trans_pcie_reclaim,
    Iwl_Trans_Pcie_Stop_Device,
    Iwl_Trans_Write8,
    Iwl_Trans_Pcie_Txq_Enable,
    iwl_trans_pcie_txq_disable,
    Iwl_Trans_Pcie_Write32,
    Iwl_Trans_Pcie_Read32,
    IwlTransPcieReadPrph,
    IwlTransPcieWritePrph,
    Iwl_Trans_Pcie_Read_Mem,
    Iwl_Trans_Pcie_Write_Mem,
    Iwl_Trans_Pcie_Configure,
    IwlTransPcieGrabNicAccess,
    IwlTransPcieReleaseNicAccess,
    Iwl_Trans_Pcie_Set_Bits_Mask,
    Dma_Map,
    Dma_Unmap,
    Iwl_Pcie_Irq_Handler,
    iwl_mvm_rs_rate_init,
    rs_alloc_sta,
    rs_tx_status,
};

INTEL_FW_PROTOCOL FwProtocol = {
        GetFirmwareInfo,
        GetFwUcodeHexLength,
        GetFwUcodeHex,
};


/**
 * @brief BiosConnectLoaderEntry
 * @details This is an entry point of Driver where it installs BiosConnectInitProtocol
 *
 * @param[In] ImageHandle
 * @param[In] SystemTable
 *
 * @retval EFI_SUCCESS upon successful return.
 *
 */

EFI_STATUS
EFIAPI
IwlTransDriverEntry(  IN  EFI_HANDLE          ImageHandle,
                         IN  EFI_SYSTEM_TABLE    *SystemTable    )
{
    EFI_STATUS  Status = EFI_SUCCESS;

    Status = gBS->InstallProtocolInterface (    &ImageHandle,
                                                &gIntelTransOpsProtocolGuid,
                                                EFI_NATIVE_INTERFACE,
                                                &PcieTransOps);

    Status = gBS->InstallProtocolInterface (    &ImageHandle,
                                                &gIntelFWProtocolGuid,
                                                EFI_NATIVE_INTERFACE,
                                                &FwProtocol);

    return Status;
}
