/**
 * @file Softmac80211Lib.h
 *
 * @details
 * This header file contains includes, defines, etc.
 * for softmac module.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 */
 
/*
 *
 * Copyright 2006-2010  Johannes Berg <johannes@sipsolutions.net>
 * Copyright 2013-2014  Intel Mobile Communications GmbH
 * Copyright 2015       Intel Deutschland GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef SOFTMAC82011LIB_H_
#define SOFTMAC82011LIB_H_

#include <UEFI.h>

#define WIFI_VHIGHTHRUPUT_CAPA_SHORT_GI_80               0x00000020
#define HT_MCS_MASK_LEN 10
#define ETHERNET_ADD_LENGTH           6
#define SSID_MAXIMUM_LEN              32  // Maximum length of SSID or network name in bytes
#define WIFI_BSS_ARP_ADDRESS_LIST_LEN 4
#define WIFI_NO_ACS 4

#define WIFI_WEP_IV_LENGTH        4
#define WIFI_WEP_ICV_LENGTH       4
#define WIFI_CCMP_MIC_LENGTH      8
#define WIFI_TKIP_ICV_LENGTH      4

#define WIFI_TX_DATA_DRV_DATA_LEN 40
#define WIFI_TX_DATA_RATE_DRV_DATA_LEN 24
#define WIFI_TX_RATES_MAX 4

#define WIFI_QOS_CTL_LENGTH       2
#define WIFI_HT_CTL_LENGTH        4
#define WIFI_FCTRL_FTYPE        0x000c
#define WIFI_DRV_FRATYPE_CTL  0x0004
#define WIFI_DRV_FRATYPE_DATA 0x0008
#define WIFI_FCTRL_TODS         0x0100
#define WIFI_FCTRL_FROMDS       0x0200
#define WIFI_STYPE_QOS_DATA            0x0080
#define WIFI_FCTRL_ORDER        0x8000
#define WIFI_FCTRL_MOREFRAGS    0x0400

#define WIFI_SCTL_DRV_SEQUENCE  0xFFF0
#define WIFI_DRV_SEQUENCE_TO_SN(seq) (((seq) & WIFI_SCTL_DRV_SEQUENCE) >> 4)

typedef struct _WIFI_SUPPORTING_BAND WIFI_SUPPORTING_BAND;
typedef struct _SK_BUFF SK_BUFF;

enum WIFI_BAND {
    WIFI_BAND_2GHZ,
    WIFI_BAND_5GHZ,
    WIFI_BAND_60GHZ,
    /* keep last */
    WIFI_NUM_BANDS
};

typedef struct _SK_BUFF {
    SK_BUFF *Next;
    SK_BUFF *Prev;
    UINT8   *Head;
    UINT8   *Data;
    VOID    *MappingSkb;
    UINT32  Len;
    CHAR8   CtrlBuff[48];
}SK_BUFF;

#pragma pack(1)
typedef struct _WIFI_Frame_Header
{
    UINT16 FrameControl;
    UINT16 Ieee80211Dur;
    UINT8  Ieee80211Addr1[6];
    UINT8  Ieee80211Addr2[6];
    UINT8  Ieee80211Addr3[6];
    UINT16 Ieee80211Seq;
    /* possibly followed by addr4[IEEE80211_ADDR_LEN]; */
    /* see below */
} WIFI_Frame_Header;
#pragma pack()

typedef struct _WIFI_CHANNEL
{
    enum WIFI_BAND    Band;
    enum nl80211_dfs_state DfsState;

    UINT16  CenterFreq;
    UINT16  HwValue;
    UINT32  Flags;
    INTN    MaxAntennaGain;
    INTN    MaxPower;
    INTN    MaxRegPower;
    BOOLEAN BeaconFound;
    UINT32  OrigFlags;
    INTN    OrigMag;
    INTN    OrigMpwr;
    UINT64  DfsStateEntered;
    UINTN   DfsCacMs;
}WIFI_CHANNEL;

typedef struct
{
    WIFI_CHANNEL            *Chan;
    enum WIFI_CHANNEL_WIDTH Width;
    UINT32                       PrimaryFrequency;
    UINT32                       SecondaryFrequency;
} WIFI_CHANNEL_DETAILS;

typedef struct
{
    UINT32  Flags;
    INTN    PowerLevel;
    INTN    DynamicPsTimeout;
    INTN    MaxSleepPeriod;
    UINT16  ListenInterval;
    UINT8   PsDTimPeriod;
    UINT8   LongFrameMaxTxCount;
    UINT8   ShortFrameMaxTxCount;
    BOOLEAN RadarEnabled;

    WIFI_CHANNEL_DETAILS ChanDef;
    enum wifi_smpsmode   SmpsMode;
} WIFI_STA_CONF;

typedef struct
{
    /* assign these fields before you register the wiphy */
#define WIPHY_COMPAT_PAD_SIZE   2048

    UINT8 Padding[WIPHY_COMPAT_PAD_SIZE];
    UINT8 PermAddr[ETHERNET_ADD_LENGTH];
    UINT8 AddrMask[ETHERNET_ADD_LENGTH];

    struct mac_address         *Addresses;
    WIFI_SUPPORTING_BAND  *Bands[WIFI_NUM_BANDS];
    enum wifi_signal_type SignalType;

    const UINT32 *CipherSuites;
    const UINT8  *ExtendedCapabilities;
    const UINT8  *ExtendedCapabilitiesMask;

    INTN   NumOfIfaceCombinations;
    UINT16 SoftwareIftypes;
    UINT16 NumOfAddresses;
    /* Supported interface modes, OR together BIT(NL80211_IFTYPE_...) */
    UINT16 InterfaceModes;
    UINT16 MaxAclMacAddrs;
    UINT32 Flags;
    UINT32 RegulatoryFlags;
    UINT32 Features;
    UINT32 ApSmeCapa;
    INTN   BssPrivSize;
    UINT8  MaxScanSsids;
    UINT8  MaxSchedScanSsids;
    UINT8  MaxMatchSets;
    UINT16 MaxScanIeLen;
    UINT16 MaxSchedScanIeLen;
    INTN   NumOfCipherSuites;
    UINT8  RetryShort;
    UINT8  RetryLong;
    UINT32 FragThreshold;
    UINT32 RtsThreshold;
    UINT8  CoverageClass;
    CHAR8  FwVersion[ETHTOOL_FWVERS_LEN];
    UINT32 HwVersion;
    UINT16 MaxRemainOnChannelDuration;
    UINT8  MaxNumPmkids;
    UINT32 AvailableAntennasTx;
    UINT32 AvailableAntennasRx;
    UINT32 ProbeRespOffload;
    UINT8  ExtendedCapabilitiesLen;
    UINT16 MaxApAssocSta;

    /* If multiple wiphys are registered and you're handed e.g.
     * a regular netdev with assigned ieee80211_ptr, you won't
     * know whether it points to a wiphy your driver has registered
     * or not. Assign this to something global to your driver to
     * help determine whether you own this wiphy or not. */
    const VOID *Privid;
    /* protects ->resume, ->suspend sysfs callbacks against unregister hw */
    BOOLEAN Registered;
} WIPHY;

typedef struct _WIFI_HARDWARE
{
    WIFI_STA_CONF Conf;
    WIPHY              *Wiphy;
    const CHAR8        *RateControlAlgorithm;

    VOID    *Priv;
    UINT32  Flags;
    INTN    VifDataSize;
    INTN    StaDataSize;
    INTN    ChanctxDataSize;
    UINT16  Queues;
    UINT16  MaxListenInterval;
    UINT8   MaxRates;
    UINT8   MaxRateTries;
    UINT8   MaxReportRates;
    UINT8   OffchannelTxHwQueue;
    UINT8   RadiotapMcsDetails;
    UINT16  RadiotapVhtDetails;
    BOOLEAN UnifiedScanSupport;
}WIFI_HARDWARE;

#pragma pack(1)
typedef struct  {
    INT8 Idx;
    UINT16 Count : 5;
    UINT16 Flags : 11;
}WIFI_TX_RATE;
#pragma pack()


struct WIFI_RATE
{
    UINT32 Flags;
    UINT16 Bitrate;
    UINT16 HwValue, HwValueShort;
};

typedef struct _WIFI_KEY_CONF
{
    UINT32 Cipher;
    UINT8  IcvLen;
    UINT8  IvLen;
    UINT8  HwKeyIdx;
    UINT8  Flags;
    INT8   KeyIdx;
    UINT8  KeyLen;
    UINT8  Key[32];
}WIFI_KEY_CONF;

struct WIFI_CHANNEL_CONTEXTCONF
{
    WIFI_CHANNEL_DETAILS Def;
    WIFI_CHANNEL_DETAILS MinDef;

    UINT8   RxChainsStatic;
    UINT8   RxChainsDynamic;
    BOOLEAN RadarEnabled;
    UINT8   DrvPriv[2];
};

#pragma pack(1)
typedef struct
{
    UINT8  Count;
    UINT32 Duration;
    UINT32 Interval;
    UINT32 StartTime;
}WIFI_P2P_DESCRICPTION;
#pragma pack()

#define WIFI_P2P_MAX_DESCRIPTION 4

#pragma pack(1)
typedef struct
{
    UINT8                      Index;
    UINT8                      OpppsCTWindow;
    WIFI_P2P_DESCRICPTION Desc[WIFI_P2P_MAX_DESCRIPTION];
} WIFI_P2P_ATTRIBUTE;

#pragma pack()

typedef struct
{
    UINT8 Bssid[6];

    /// association related data
    BOOLEAN Assoc;
    BOOLEAN IbssJoined;
    BOOLEAN IbssCreator;
    UINT16  Aid;

    /// erp related data
    BOOLEAN UseCtsProt;
    BOOLEAN UseShortPreamble;
    BOOLEAN UseShortSlot;
    BOOLEAN EnableBeacon;
    UINT8   DtimPeriod;
    UINT16  BeaconInt;
    UINT16  AssocCapability;
    UINT64  SyncTsf;
    UINT32  SyncDeviceTs;
    UINT8   SyncDtimCount;
    UINT32  BasicRates;
    INTN    McastRate[WIFI_NUM_BANDS];
    UINT16  HtOperationMode;
    INT32   CqmRssiThold;
    UINT32  CqmRssiHyst;
    UINT32  ArpAddrList[WIFI_BSS_ARP_ADDRESS_LIST_LEN];
    INTN    ArpAddrCnt;
    BOOLEAN Qos;
    BOOLEAN Idle;
    BOOLEAN Ps;
    UINT8   Ssid[SSID_MAXIMUM_LEN];
    size_t  SsidLen;
    BOOLEAN HiddenSsid;
    INTN    TxPower;

    struct WIFI_RATE     *BeaconRate;
    WIFI_CHANNEL_DETAILS Chandef;
    WIFI_P2P_ATTRIBUTE   P2pNoaAttr;
}WIFI_BSS_CONF;

typedef struct
{
    enum wifi_interface_type Type;
    WIFI_BSS_CONF            BssConf;

    UINT8   Addr[ETHERNET_ADD_LENGTH];
    BOOLEAN P2p;
    BOOLEAN CsaActive;
    UINT8   CabQueue;
    UINT8   HwQueue[WIFI_NO_ACS];
    UINT32  DriverFlags;

    struct WIFI_CHANNEL_CONTEXTCONF  *ChanctxConf;
    /* must be last */
#pragma pack(8)
    UINT8 DrvPriv[392];
#pragma pack()
}WIFI_VIF;

typedef struct
{
    /* common information */
    UINT32 Flags;
    UINT8  Band;
    UINT8  HwQueue;
    UINT16 AckFrameId;

    union {
        struct {
            union {
                /* rate control */
                struct {
                    WIFI_TX_RATE Rates[WIFI_TX_RATES_MAX];
                    INT8              RtsCtsRateIdx;
                    UINT8             UseRts : 1;
                    UINT8             UseCtsProt : 1;
                    UINT8             ShortPreamble : 1;
                    UINT8             SkipTable : 1;
                    /* 2 bytes free */
                } WIFI_RATEINFO;
                /* only needed before rate control */
                UINT64 Jiffies;
            } RATE_CONTROL;
            /* NB: vif can be NULL for injected frames */
            WIFI_VIF      *Vif;
            WIFI_KEY_CONF *HwKey;
            UINT32             Flags;
            /* 4 bytes free */
        } CONTROL;
        struct {
            WIFI_TX_RATE Rates[WIFI_TX_RATES_MAX];
            INT32             AckSignal;
            UINT8             AmpduAckLen;
            UINT8             AmpduLen;
            UINT8             Antenna;
            VOID              *StatusDriverData[21 / sizeof(VOID *)];
        } STATUS;
        struct {
            WIFI_TX_RATE DriverRates[WIFI_TX_RATES_MAX];
            UINT8             Pad[4];
            VOID              *RateDriverData[WIFI_TX_DATA_RATE_DRV_DATA_LEN / sizeof(VOID *)];
        } DRIVER_RATE;
        VOID *DriverData[WIFI_TX_DATA_DRV_DATA_LEN / sizeof(VOID *)];
    } TX_INFO;

}WIFI_TX_DATA;

enum wifi_rate_ctrl_flags
{
    WIFI_TX_RATE_CTL_USE_RTS_CTS_EX     = BIT(0),
    WIFI_TX_RATE_CTL_CTS_SELF_PROTECT   = BIT(1),
    WIFI_TX_RATE_CTL_SHORT_PREAMBLE_USE = BIT(2),
    WIFI_TX_RATE_CTL_HT_MCS             = BIT(3),
    WIFI_TX_RATE_CTL_GREEN_FIELD_MODE   = BIT(4),
    WIFI_TX_RATE_CTL_WIDTH_40_MHZ       = BIT(5),
    WIFI_TX_RATE_CTL_DUP_FRAME          = BIT(6),
    WIFI_TX_RATE_CTL_USE_SHORT_GI       = BIT(7),
    WIFI_TX_RATE_CTL_VHT_MCS            = BIT(8),
    WIFI_TX_RATE_CTL_WIDTH_80_MHZ       = BIT(9),
    WIFI_TX_RATE_CTL_WIDTH_160_MHZ      = BIT(10),
};

#pragma pack(1)
typedef struct
{
    UINT8  RxMask[HT_MCS_MASK_LEN];
    UINT16 RxHighestDataRate;
    UINT8  TxParams;
    UINT8  Reserved[3];
} WIFI_MCS_DATA;

#pragma pack(1)
typedef struct
{
    UINT16 RxMCSMap;
    UINT16 RxHighestDataRate;
    UINT16 TxMCSMap;
    UINT16 TxHighestDataRate;
} WIFI_VHT_MCS_INFO;
#pragma pack()

typedef struct
{
    UINT16             Cap;         /// use IEEE80211_HT_CAP_
    BOOLEAN            HTSupported;
    UINT8              AmpduFactor;
    UINT8              AmpduDensity;
    WIFI_MCS_DATA MCS;
} WIFI_STATIONHTCAP;

typedef struct
{
    BOOLEAN               VHTSupported;
    UINT32                Cap;     // use IEEE80211_VHT_CAP_
    WIFI_VHT_MCS_INFO VHTMCS;
} WIFI_STATIONVHTCAP;

typedef struct _WIFI_SUPPORTING_BAND
{
    WIFI_CHANNEL       *Channels;
    struct WIFI_RATE   *Bitrates;
    enum WIFI_BAND     Band;
    INTN                    NumOfChannels;
    INTN                    NumOfBitrates;
    WIFI_STATIONHTCAP  HtCap;
    WIFI_STATIONVHTCAP VhtCap;
}WIFI_SUPPORTING_BAND;

enum wifi_rx_station_bandwidth
{
    WIFI_RX_STATION_BW_20 = 0,
    WIFI_RX_STATION_BW_40,
    WIFI_RX_STATION_BW_80,
    WIFI_RX_STATION_BW_160,
};

typedef struct
{
    UINT32 SuppRates[WIFI_NUM_BANDS];
    UINT8  Addr[ETHERNET_ADD_LENGTH];
    UINT16 Aid;

    WIFI_STATIONHTCAP  HtCap;
    WIFI_STATIONVHTCAP VhtCap;

    BOOLEAN Wme;
    BOOLEAN Tdls;
    UINT8   UapsdQueues;
    UINT8   MaxSp;
    UINT8   RxNss;

    enum wifi_rx_station_bandwidth Bandwidth;
    enum wifi_smpsmode             SmpsMode;
    struct ieee80211_sta_rates      *Rates;
#pragma pack(8)
    UINT8 DrvPriv[1280];
#pragma pack()
}WIFI_STATION;

STATIC INTN IsDataFrame(UINT16 FC)
{
    return (FC & (CONV_TO_LE16_BIT(WIFI_FCTRL_FTYPE))) ==
           (CONV_TO_LE16_BIT(WIFI_DRV_FRATYPE_DATA));
}

STATIC INTN CheckTodsFromdsFlags(UINT16 FC)
{
    UINT16 Temp = (CONV_TO_LE16_BIT(WIFI_FCTRL_TODS | WIFI_FCTRL_FROMDS));
    return (FC & Temp) == Temp;
}
STATIC INTN CheckQosData(UINT16 FC)
{
    /*
     * mask with QOS_DATA rather than WIFI_FCTRL_STYPE as we just need
     * to check the one bit
     */
    return (FC & (CONV_TO_LE16_BIT(WIFI_FCTRL_FTYPE | WIFI_STYPE_QOS_DATA))) ==
           (CONV_TO_LE16_BIT(WIFI_DRV_FRATYPE_DATA | WIFI_STYPE_QOS_DATA));
}

STATIC INTN CheckOrderFlag(UINT16 FC)
{
    return (FC & (CONV_TO_LE16_BIT(WIFI_FCTRL_ORDER))) != 0;
}

STATIC INTN IsControlFrame(UINT16 FC)
{
    return (FC & (CONV_TO_LE16_BIT(WIFI_FCTRL_FTYPE))) ==
           (CONV_TO_LE16_BIT(WIFI_DRV_FRATYPE_CTL));
}

STATIC INTN CheckMoreFragsFlag(UINT16 FC)
{
    return (FC & CONV_TO_LE16_BIT(WIFI_FCTRL_MOREFRAGS)) != 0;
}

STATIC UINTN GetHeaderLength(
    UINT16 FC
    )
{
    UINTN HeaderLen = 24;

    if (IsDataFrame(FC))
    {
        if (CheckTodsFromdsFlags(FC))
        {
            HeaderLen = 30;
        }
        if (CheckQosData(FC))
        {
            HeaderLen += WIFI_QOS_CTL_LENGTH;
            if (CheckOrderFlag(FC))
            {
                HeaderLen += WIFI_HT_CTL_LENGTH;
            }
        }
        goto out;
    }

    if (IsControlFrame(FC))
    {
        if ((FC & CONV_TO_LE16_BIT(0x00E0)) == CONV_TO_LE16_BIT(0x00C0))
        {
            HeaderLen = 10;
        }
        else
        {
            HeaderLen = 16;
        }
    }
out:
    return HeaderLen;
}

enum wifi_interface_type
{
    WIFI_INTERFACE_UNSPECIFIED,
    WIFI_INTERFACE_ADHOC,
    WIFI_INTERFACE_STA,
    WIFI_INTERFACE_AP,
    WIFI_INTERFACE_AP_VLAN,
    WIFI_INTERFACE_WDS,
    WIFI_INTERFACE_MONITOR,
    WIFI_INTERFACE_MESH_POINT,
    WIFI_INTERFACE_P2P_CLIENT,
    WIFI_INTERFACE_P2P_GO,
    WIFI_INTERFACE_P2P_DEVICE,
    WIFI_NUM_INTERFACE_TYPES,
    WIFI_INTERFACE_MAX = WIFI_NUM_INTERFACE_TYPES - 1
};

enum wifi_tx_info_flags
{
    WIFI_TX_CTRL_ASSIGN_SEQ_NO         = BIT(1),
    WIFI_TX_CTRL_NO_ACK_WAIT           = BIT(2),
    WIFI_TX_CTRL_CLR_PS_FILTER         = BIT(3),
    WIFI_TX_CTRL_FIRST_FRAME_FRAGMENT  = BIT(4),
    WIFI_TX_CTRL_SEND_FRAME_AFTER_DTIM = BIT(5),
    WIFI_TX_CTRL_FRAME_WITH_AMPDU      = BIT(6),
    WIFI_TX_CTRL_FRAME_INJECTED        = BIT(7),
    WIFI_TX_INFO_TX_FILTERED           = BIT(8),
    WIFI_TX_FRAME_ACK                  = BIT(9),
    WIFI_TX_DATA_AMPDU                 = BIT(10),
    WIFI_TX_DATA_AMPDU_NO_BLK_ACK      = BIT(11),
    WIFI_TX_RATE_CTRL_PROBE            = BIT(12),
    WIFI_TX_CTRL_USE_LDPC              = BIT(22),
    WIFI_TX_CTRL_ENABLE_STBC           = BIT(23) | BIT(24),
    WIFI_TX_CTRL_TX_FRAME_OFF_CHANNEL  = BIT(25),
    WIFI_TX_CTRL_FRAME_NOCCK_RATE      = BIT(27),
    WIFI_TX_STATUS_SERVICE_PERIOD_ENDS = BIT(28),
    WIFI_TX_INFO_NO_ACK_TRANSMITTED    = BIT(31),

};

#endif /* SOFTMAC82011LIB_H_ */
