/**
 * @file TransType.h
 *
 * @details
 * This header file contains function prototypes, defines, etc.
 * for UNDI driver.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2017 Dell Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * BSD LICENSE
 *
 * Copyright(c) 2007 Dell Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Dell Inc nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef TRANSTYPES_H_
#define TRANSTYPES_H_
/******************************************************************/
/***
*			                MACROS
*/
/******************************************************************/
#define CONV_TO_LE32_BIT(x)  x
#define le32_to_cpu(x) x
#define CONV_TO_LE16_BIT(x) x
#define le16_to_cpu(x) x

#define max_t(type, x, y)  ((x) > (y) ? (x) : (y))
#define min_t(type, x, y)  ((x) < (y) ? (x) : (y))
#define Noffsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)

#define DIV_ROUND_UP(n, d) (((n) + (d) - 1) / (d))
#define ARRAY_SIZEOF(x) (sizeof(x) / sizeof(x[0]))

#define max(x, y) ({                \
                       typeof(x)_max1 = (x);          \
                       typeof(y)_max2 = (y);          \
                       (void) (&_max1 == &_max2);      \
                       _max1 > _max2 ? _max1 : _max2; })


#define roundup(x, y) ((((x)+((y)-1))/(y))*(y))
#define rounddown(x, y)  (((x)/(y))*(y))

#define __ALIGN_KERNEL_MASK(x, mask)    (((x) + (mask)) & ~(mask))

#define __ALIGN_KERNEL(x, a)        __ALIGN_KERNEL_MASK(x, ((a) - 1))

#define ALIGN(x, a)     __ALIGN_KERNEL((x), (a))
#define IS_ALIGNED(x, a)        (((x) & ((typeof(x))(a) - 1)) == 0)

#define BUILD_BUG_ON(condition) ((void)sizeof(UINT8[1 - 2*!!(condition)]))
#define WARN_ON(x) (x)
#define WARN_ON_ONCE(x) (x)
#define likely(x) (x)

#define __u32  UINT32
#define __u64  UINT64
#define __u8  UINT8
#define __u16  UINT16

#define __le64 UINT64
#define __le32 UINT32
#define __le16 UINT16
#define __le8 UINT8
#define __s8 INT8
#define __s16 INT16


#define __be32 UINT32
#define __be16 UINT16

#define s8 INT8
#define s32 INT32
#define s64 INT64
#define u8 UINT8
#define u16 UINT16
#define u32 UINT32
#define u64 UINT64

#define bool UINT8

#define BIT(x)  (1 << (x))

typedef __s16 int16_t;

static void le32_add_cpu(__le32 *var, u32 val)
{
    *var = CONV_TO_LE32_BIT(le32_to_cpu(*var) + val);
}

#define true 1
#define false 0

INT32
test_bit (
    IN UINT64 nr,
    IN const volatile void *addr
    );

INT32
test_and_set_bit (
    IN UINT64 nr,
    IN unsigned long *addr
    );
INT32
test_and_clear_bit (
    IN UINT64 nr,
    IN unsigned long  *addr
    );
VOID
clear_bit (
    INTN nr,
    unsigned long *addr
    );

VOID
set_bit (
    UINT64 nr,
    unsigned long *addr
    );

#define true 1
#define false 0

#define DIV_ROUND_UP(n, d) (((n) + (d) - 1) / (d))

#define BITS_PER_BYTE           8
#define BITS_TO_LONGS(nr)       DIV_ROUND_UP(nr, BITS_PER_BYTE * sizeof(long))
#define BITS_TO_U64(nr)         DIV_ROUND_UP(nr, BITS_PER_BYTE * sizeof(u64))
#define BITS_TO_U32(nr)         DIV_ROUND_UP(nr, BITS_PER_BYTE * sizeof(u32))
#define BITS_TO_BYTES(nr)       DIV_ROUND_UP(nr, BITS_PER_BYTE)

struct list_head {
    struct list_head *next, *prev;
};
#define DECLARE_BITMAP(name, bits) \
    unsigned long name[BITS_TO_LONGS(bits)]
#endif /* TRANSTYPEs_H_ */
