/**
 * @file iwl-config.h
 *
 * @details
 * This header file contains macros for Antenna presence definitions
 * for Intel UNDI driver.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2012 - 2014 Intel Corporation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * Contact Information:
 *  Intel Linux Wireless <ilw@linux.intel.com>
 * Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 *
 * BSD LICENSE
 *
 * Copyright(c) 2012 - 2014 Intel Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Intel Corporation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef IWLCONFIG_H_
#define IWLCONFIG_H_

//Macros

/* Antenna presence definitions */
#define ANT_A       BIT(0)
#define ANT_B       BIT(1)
#define ANT_C       BIT(2)
#define ANT_AB      (ANT_A | ANT_B)
#define ANT_AC      (ANT_A | ANT_C)
#define ANT_BC      (ANT_B | ANT_C)
#define ANT_ABC     (ANT_A | ANT_B | ANT_C)

typedef enum iwl_device_family {
    IWL_DEVICE_FAMILY_UNDEFINED,
    IWL_DEVICE_FAMILY_1000,
    IWL_DEVICE_FAMILY_100,
    IWL_DEVICE_FAMILY_2000,
    IWL_DEVICE_FAMILY_2030,
    IWL_DEVICE_FAMILY_105,
    IWL_DEVICE_FAMILY_135,
    IWL_DEVICE_FAMILY_5000,
    IWL_DEVICE_FAMILY_5150,
    IWL_DEVICE_FAMILY_6000,
    IWL_DEVICE_FAMILY_6000i,
    IWL_DEVICE_FAMILY_6005,
    IWL_DEVICE_FAMILY_6030,
    IWL_DEVICE_FAMILY_6050,
    IWL_DEVICE_FAMILY_6150,
    IWL_DEVICE_FAMILY_7000,
    IWL_DEVICE_FAMILY_8000,
}IWL_DEVICE_FAMILY;


typedef struct iwl_eeprom_params {
    CONST UINT8 Regulatory_Bands[7];
    BOOLEAN Enhanced_Txpower;
}IWL_EEPROM_PARAMS;

/*
 * @stbc: support Tx STBC and 1*SS Rx STBC
 * @use_rts_for_aggregation: use rts/cts protection for HT traffic
 * @ht40_bands: bitmap of bands (using %IEEE80211_BAND_*) that support HT40
 */
typedef struct iwl_ht_params {

    CONST BOOLEAN Ht_Greenfield_Support; /* if used set to true */
    BOOLEAN Stbc;
    BOOLEAN Use_Rts_For_Aggregation;
    UINT8 Ht40_Bands;
}IWL_HT_PARAMS;

/*
 * @max_ll_items: max number of OTP blocks
 * @shadow_ram_support: shadow support for OTP memory
 * @led_compensation: compensate on the led on/off time per HW according
 *	to the deviation to achieve the desired led frequency.
 *	The detail algorithm is described in iwl-led.c
 * @wd_timeout: TX queues watchdog timeout
 * @max_event_log_size: size of event log buffer size for ucode event logging
 * @shadow_reg_enable: HW shadow register support
 */
typedef struct iwl_base_params {
    INT32 Eeprom_Size;
    INT32 Num_Of_Queues;    /* def: HW dependent */
    /* for iwl_pcie_apm_init() */
    UINT32 Pll_Cfg_Val;

    CONST UINT16 Max_ll_Items;
    CONST BOOLEAN Shadow_Ram_Support;
    UINT16 Led_Compensation;
    UINT32 Wd_Timeout;
    UINT32 Max_Event_Log_Size;
    CONST BOOLEAN Shadow_Reg_Enable;
    CONST BOOLEAN Pcie_l1_Allowed;
}IWL_BASE_PARAMS;



/**
 * struct iwl_cfg
 * @name: Offical name of the device
 * @fw_name_pre: Firmware filename prefix. The api version and extension
 *	(.ucode) will be added to filename before loading from disk. The
 *	filename is constructed as fw_name_pre<api>.ucode.
 * @ucode_api_max: Highest version of uCode API supported by driver.
 * @ucode_api_ok: oldest version of the uCode API that is OK to load
 *	without a warning, for use in transitions
 * @ucode_api_min: Lowest version of uCode API supported by driver.
 * @max_inst_size: The maximal length of the fw inst section
 * @max_data_size: The maximal length of the fw data section
 * @valid_tx_ant: valid transmit antenna
 * @valid_rx_ant: valid receive antenna
 * @nvm_ver: NVM version
 * @nvm_calib_ver: NVM calibration version
 * @lib: pointer to the lib ops
 * @base_params: pointer to basic parameters
 * @ht_params: point to ht parameters
 * @led_mode: 0=blinking, 1=On(RF On)/Off(RF Off)
 * @rx_with_siso_diversity: 1x1 device with rx antenna diversity
 * @internal_wimax_coex: internal wifi/wimax combo device
 * @high_temp: Is this NIC is designated to be in high temperature.
 * @host_interrupt_operation_mode: device needs host interrupt operation
 *	mode set
 * @d0i3: device uses d0i3 instead of d3
 * @nvm_hw_section_num: the ID of the HW NVM section
 * @pwr_tx_backoffs: translation table between power limits and backoffs
 *
 * We enable the driver to be backward compatible wrt. hardware features.
 * API differences in uCode shouldn't be handled here but through TLVs
 * and/or the uCode API version instead.
 */
typedef struct iwl_cfg {
    /* params specific to an individual device within a device family */
    CONST CHAR8 *Name;
    CONST CHAR8 *Fw_Name_Pre;
    UINT32 Ucode_Api_Max;
    UINT32 Ucode_Api_Ok;
    UINT32 Ucode_Api_Min;
    IWL_DEVICE_FAMILY Device_Family;
    UINT32 Device_Id;
    CONST UINT32 Max_Data_Size;
    CONST UINT32 Max_Inst_Size;
    UINT8 Valid_Tx_Ant;
    UINT8 Valid_Rx_Ant;
    BOOLEAN Bt_Shared_Single_Ant;
    UINT16 Nvm_Ver;
    UINT16 Nvm_Calib_Ver;
    /* params not likely to change within a device family */
    IWL_BASE_PARAMS *Base_Params;
    /* params likely to change within a device family */
    IWL_HT_PARAMS *Ht_Params;
    CONST IWL_EEPROM_PARAMS *Eeprom_Params;
    enum iwl_led_mode Led_Mode;
    CONST BOOLEAN Rx_With_Siso_Diversity;
    CONST BOOLEAN Internal_Wimax_Coex;
    CONST BOOLEAN Host_Interrupt_Operation_Mode;
    BOOLEAN High_Temp;
    BOOLEAN D0i3;
    UINT8 Nvm_Hw_Section_Num;
    BOOLEAN Lp_Xtal_Workaround;
    CONST struct iwl_pwr_tx_backoff *Pwr_Tx_Backoffs;
    CONST char *Default_Nvm_File_B_Step;
    CONST char *Default_Nvm_File_C_Step;
    BOOLEAN Vht_MU_MIMO_Supported;
}IWL_CONFIG;


#endif
