/**
 * @file iwl-fw.h
 *
 * @details
 * This header file contains structures and enums related to ucode
 * and fw for Intel UNDI driver.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2012 - 2014 Intel Corporation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * Contact Information:
 *  Intel Linux Wireless <ilw@linux.intel.com>
 * Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 *
 * BSD LICENSE
 *
 * Copyright(c) 2012 - 2014 Intel Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Intel Corporation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef IWLFW_H_
#define IWLFW_H_

#define BIT(x)  (1 << (x))

#define DIV_ROUND_UP(n,d) (((n) + (d) - 1) / (d))

#define BITS_PER_BYTE           8
#define BITS_TO_LONGS(nr)       DIV_ROUND_UP(nr, BITS_PER_BYTE * sizeof(long))

#define ETHTOOL_FWVERS_LEN  32  //DEFINED IN ETHTOOL.H. FOR TIME BEING COPIED HERE
#define IWL_UCODE_SECTION_MAX 16


/**
 * enum iwl_ucode_tlv_flag - ucode API flags
 * @IWL_UCODE_TLV_FLAGS_PAN: This is PAN capable microcode; this previously
 *	was a separate TLV but moved here to save space.
 * @IWL_UCODE_TLV_FLAGS_NEWSCAN: new uCode scan behaviour on hidden SSID,
 *	treats good CRC threshold as a boolean
 * @IWL_UCODE_TLV_FLAGS_MFP: This uCode image supports MFP (802.11w).
 * @IWL_UCODE_TLV_FLAGS_P2P: This uCode image supports P2P.
 * @IWL_UCODE_TLV_FLAGS_DW_BC_TABLE: The SCD byte count table is in DWORDS
 * @IWL_UCODE_TLV_FLAGS_UAPSD: This uCode image supports uAPSD
 * @IWL_UCODE_TLV_FLAGS_SHORT_BL: 16 entries of black list instead of 64 in scan
 *	offload profile config command.
 * @IWL_UCODE_TLV_FLAGS_RX_ENERGY_API: supports rx signal strength api
 * @IWL_UCODE_TLV_FLAGS_TIME_EVENT_API_V2: using the new time event API.
 * @IWL_UCODE_TLV_FLAGS_D3_6_IPV6_ADDRS: D3 image supports up to six
 *	(rather than two) IPv6 addresses
 * @IWL_UCODE_TLV_FLAGS_BF_UPDATED: new beacon filtering API
 * @IWL_UCODE_TLV_FLAGS_NO_BASIC_SSID: not sending a probe with the SSID element
 *	from the probe request template.
 * @IWL_UCODE_TLV_FLAGS_D3_CONTINUITY_API: modified D3 API to allow keeping
 *	connection when going back to D0
 * @IWL_UCODE_TLV_FLAGS_NEW_NSOFFL_SMALL: new NS offload (small version)
 * @IWL_UCODE_TLV_FLAGS_NEW_NSOFFL_LARGE: new NS offload (large version)
 * @IWL_UCODE_TLV_FLAGS_SCHED_SCAN: this uCode image supports scheduled scan.
 * @IWL_UCODE_TLV_FLAGS_STA_KEY_CMD: new ADD_STA and ADD_STA_KEY command API
 * @IWL_UCODE_TLV_FLAGS_DEVICE_PS_CMD: support device wide power command
 *	containing CAM (Continuous Active Mode) indication.
 * @IWL_UCODE_TLV_FLAGS_P2P_BSS_PS_DCM: support power save on BSS station and
 *	P2P client interfaces simultaneously if they are in different bindings.
 * @IWL_UCODE_TLV_FLAGS_P2P_PS_UAPSD: P2P client supports uAPSD power save
 * @IWL_UCODE_TLV_FLAGS_BCAST_FILTERING: uCode supports broadcast filtering.
 * @IWL_UCODE_TLV_FLAGS_GO_UAPSD: AP/GO interfaces support uAPSD clients
 */

enum iwl_ucode_tlv_flag {
    IWL_UCODE_TLV_FLAGS_PAN         = BIT(0),
    IWL_UCODE_TLV_FLAGS_NEWSCAN     = BIT(1),
    IWL_UCODE_TLV_FLAGS_MFP         = BIT(2),
    IWL_UCODE_TLV_FLAGS_P2P         = BIT(3),
    IWL_UCODE_TLV_FLAGS_DW_BC_TABLE     = BIT(4),
    IWL_UCODE_TLV_FLAGS_NEWBT_COEX      = BIT(5),
    IWL_UCODE_TLV_FLAGS_PM_CMD_SUPPORT  = BIT(6),
    IWL_UCODE_TLV_FLAGS_SHORT_BL        = BIT(7),
    IWL_UCODE_TLV_FLAGS_RX_ENERGY_API   = BIT(8),
    IWL_UCODE_TLV_FLAGS_TIME_EVENT_API_V2   = BIT(9),
    IWL_UCODE_TLV_FLAGS_D3_6_IPV6_ADDRS = BIT(10),
    IWL_UCODE_TLV_FLAGS_BF_UPDATED      = BIT(11),
    IWL_UCODE_TLV_FLAGS_NO_BASIC_SSID   = BIT(12),
    IWL_UCODE_TLV_FLAGS_D3_CONTINUITY_API   = BIT(14),
    IWL_UCODE_TLV_FLAGS_NEW_NSOFFL_SMALL    = BIT(15),
    IWL_UCODE_TLV_FLAGS_NEW_NSOFFL_LARGE    = BIT(16),
    IWL_UCODE_TLV_FLAGS_SCHED_SCAN      = BIT(17),
    IWL_UCODE_TLV_FLAGS_STA_KEY_CMD     = BIT(19),
    IWL_UCODE_TLV_FLAGS_DEVICE_PS_CMD   = BIT(20),
    IWL_UCODE_TLV_FLAGS_BSS_P2P_PS_DCM  = BIT(22),
    IWL_UCODE_TLV_FLAGS_UAPSD_SUPPORT   = BIT(24),
    IWL_UCODE_TLV_FLAGS_P2P_PS_UAPSD    = BIT(26),
    IWL_UCODE_TLV_FLAGS_BCAST_FILTERING = BIT(29),
    IWL_UCODE_TLV_FLAGS_GO_UAPSD        = BIT(30),
};

/**
 * enum iwl_ucode_type
 *
 * The type of ucode.
 *
 * @IWL_UCODE_REGULAR: Normal runtime ucode
 * @IWL_UCODE_INIT: Initial ucode
 * @IWL_UCODE_WOWLAN: Wake on Wireless enabled ucode
 * @IWL_UCODE_REGULAR_USNIFFER: Normal runtime ucode when using usniffer image
 */
enum iwl_ucode_type {
    IWL_UCODE_REGULAR,
    IWL_UCODE_INIT,
    IWL_UCODE_WOWLAN,
    IWL_UCODE_REGULAR_USNIFFER,
    IWL_UCODE_TYPE_MAX,
};

/**
 * enum iwl_ucode_tlv_api - ucode api
 * @IWL_UCODE_TLV_API_BT_COEX_SPLIT: new API for BT Coex
 * @IWL_UCODE_TLV_API_FRAGMENTED_SCAN: This ucode supports active dwell time
 *	longer than the passive one, which is essential for fragmented scan.
 * @IWL_UCODE_TLV_API_WIFI_MCC_UPDATE: ucode supports MCC updates with source.
 * @IWL_UCODE_TLV_API_WIDE_CMD_HDR: ucode supports wide command header
 * @IWL_UCODE_TLV_API_LQ_SS_PARAMS: Configure STBC/BFER via LQ CMD ss_params
 * @IWL_UCODE_TLV_API_NEW_VERSION: new versioning format
 * @IWL_UCODE_TLV_API_EXT_SCAN_PRIORITY: scan APIs use 8-level priority
 *	instead of 3.
 * @IWL_UCODE_TLV_API_TX_POWER_CHAIN: TX power API has larger command size
 *	(command version 3) that supports per-chain limits
 *
 * @NUM_IWL_UCODE_TLV_API: number of bits used
 */
enum iwl_ucode_tlv_api {
    IWL_UCODE_TLV_API_BT_COEX_SPLIT         = 3,
    IWL_UCODE_TLV_API_FRAGMENTED_SCAN   = 8,
    IWL_UCODE_TLV_API_WIFI_MCC_UPDATE   = 9,
    IWL_UCODE_TLV_API_WIDE_CMD_HDR      = 14,
    IWL_UCODE_TLV_API_LQ_SS_PARAMS      = 18,
    IWL_UCODE_TLV_API_NEW_VERSION       = 20,
    IWL_UCODE_TLV_API_EXT_SCAN_PRIORITY = 24,
    IWL_UCODE_TLV_API_TX_POWER_CHAIN    = 27,
    NUM_IWL_UCODE_TLV_API

};


/**
 * enum iwl_ucode_tlv_capa - ucode capabilities
 * @IWL_UCODE_TLV_CAPA_D0I3_SUPPORT: supports D0i3
 * @IWL_UCODE_TLV_CAPA_LAR_SUPPORT: supports Location Aware Regulatory
 * @IWL_UCODE_TLV_CAPA_UMAC_SCAN: supports UMAC scan.
 * @IWL_UCODE_TLV_CAPA_BEAMFORMER: supports Beamformer
 * @IWL_UCODE_TLV_CAPA_TOF_SUPPORT: supports Time of Flight (802.11mc FTM)
 * @IWL_UCODE_TLV_CAPA_TDLS_SUPPORT: support basic TDLS functionality
 * @IWL_UCODE_TLV_CAPA_TXPOWER_INSERTION_SUPPORT: supports insertion of current
 *	tx power value into TPC Report action frame and Link Measurement Report
 *	action frame
 * @IWL_UCODE_TLV_CAPA_DS_PARAM_SET_IE_SUPPORT: supports updating current
 *	channel in DS parameter set element in probe requests.
 * @IWL_UCODE_TLV_CAPA_WFA_TPC_REP_IE_SUPPORT: supports adding TPC Report IE in
 *	probe requests.
 * @IWL_UCODE_TLV_CAPA_QUIET_PERIOD_SUPPORT: supports Quiet Period requests
 * @IWL_UCODE_TLV_CAPA_DQA_SUPPORT: supports dynamic queue allocation (DQA),
 *	which also implies support for the scheduler configuration command
 * @IWL_UCODE_TLV_CAPA_TDLS_CHANNEL_SWITCH: supports TDLS channel switching
 * @IWL_UCODE_TLV_CAPA_CNSLDTD_D3_D0_IMG: Consolidated D3-D0 image
 * @IWL_UCODE_TLV_CAPA_HOTSPOT_SUPPORT: supports Hot Spot Command
 * @IWL_UCODE_TLV_CAPA_DC2DC_SUPPORT: supports DC2DC Command
 * @IWL_UCODE_TLV_CAPA_2G_COEX_SUPPORT: supports 2G coex Command
 * @IWL_UCODE_TLV_CAPA_CSUM_SUPPORT: supports TCP Checksum Offload
 * @IWL_UCODE_TLV_CAPA_RADIO_BEACON_STATS: support radio and beacon statistics
 * @IWL_UCODE_TLV_CAPA_P2P_STANDALONE_UAPSD: support p2p standalone U-APSD
 * @IWL_UCODE_TLV_CAPA_BT_COEX_PLCR: enabled BT Coex packet level co-running
 * @IWL_UCODE_TLV_CAPA_LAR_MULTI_MCC: ucode supports LAR updates with different
 *	sources for the MCC. This TLV bit is a future replacement to
 *	IWL_UCODE_TLV_API_WIFI_MCC_UPDATE. When either is set, multi-source LAR
 *	is supported.
 * @IWL_UCODE_TLV_CAPA_BT_COEX_RRC: supports BT Coex RRC
 * @IWL_UCODE_TLV_CAPA_GSCAN_SUPPORT: supports gscan
 * @IWL_UCODE_TLV_CAPA_NAN_SUPPORT: supports NAN
 * @IWL_UCODE_TLV_CAPA_EXTENDED_DTS_MEASURE: extended DTS measurement
 * @IWL_UCODE_TLV_CAPA_SHORT_PM_TIMEOUTS: supports short PM timeouts
 * @IWL_UCODE_TLV_CAPA_BT_MPLUT_SUPPORT: supports bt-coex Multi-priority LUT
 * @IWL_UCODE_TLV_CAPA_BEACON_ANT_SELECTION: firmware will decide on what
 *	antenna the beacon should be transmitted
 * @IWL_UCODE_TLV_CAPA_BEACON_STORING: firmware will store the latest beacon
 *	from AP and will send it upon d0i3 exit.
 * @IWL_UCODE_TLV_CAPA_LAR_SUPPORT_V2: support LAR API V2
 * @IWL_UCODE_TLV_CAPA_CT_KILL_BY_FW: firmware responsible for CT-kill
 * @IWL_UCODE_TLV_CAPA_TEMP_THS_REPORT_SUPPORT: supports temperature
 *	thresholds reporting
 * @IWL_UCODE_TLV_CAPA_CTDP_SUPPORT: supports cTDP command
 *
 * @NUM_IWL_UCODE_TLV_CAPA: number of bits used
 */
enum iwl_ucode_tlv_capa {
    IWL_UCODE_TLV_CAPA_D0I3_SUPPORT         = 0,
    IWL_UCODE_TLV_CAPA_LAR_SUPPORT          = 1,
    IWL_UCODE_TLV_CAPA_UMAC_SCAN            = 2,
    IWL_UCODE_TLV_CAPA_BEAMFORMER           = 3,
    IWL_UCODE_TLV_CAPA_TOF_SUPPORT                  = 5,
    IWL_UCODE_TLV_CAPA_TDLS_SUPPORT         = 6,
    IWL_UCODE_TLV_CAPA_TXPOWER_INSERTION_SUPPORT    = 8,
    IWL_UCODE_TLV_CAPA_DS_PARAM_SET_IE_SUPPORT  = 9,
    IWL_UCODE_TLV_CAPA_WFA_TPC_REP_IE_SUPPORT   = 10,
    IWL_UCODE_TLV_CAPA_QUIET_PERIOD_SUPPORT     = 11,
    IWL_UCODE_TLV_CAPA_DQA_SUPPORT          = 12,
    IWL_UCODE_TLV_CAPA_TDLS_CHANNEL_SWITCH      = 13,
    IWL_UCODE_TLV_CAPA_CNSLDTD_D3_D0_IMG        = 17,
    IWL_UCODE_TLV_CAPA_HOTSPOT_SUPPORT      = 18,
    IWL_UCODE_TLV_CAPA_DC2DC_CONFIG_SUPPORT     = 19,
    IWL_UCODE_TLV_CAPA_2G_COEX_SUPPORT      = 20,
    IWL_UCODE_TLV_CAPA_CSUM_SUPPORT         = 21,
    IWL_UCODE_TLV_CAPA_RADIO_BEACON_STATS       = 22,
    IWL_UCODE_TLV_CAPA_P2P_STANDALONE_UAPSD     = 26,
    IWL_UCODE_TLV_CAPA_BT_COEX_PLCR         = 28,
    IWL_UCODE_TLV_CAPA_LAR_MULTI_MCC        = 29,
    IWL_UCODE_TLV_CAPA_BT_COEX_RRC          = 30,
    IWL_UCODE_TLV_CAPA_GSCAN_SUPPORT        = 31,
    IWL_UCODE_TLV_CAPA_NAN_SUPPORT          = 34,
    IWL_UCODE_TLV_CAPA_EXTENDED_DTS_MEASURE     = 64,
    IWL_UCODE_TLV_CAPA_SHORT_PM_TIMEOUTS        = 65,
    IWL_UCODE_TLV_CAPA_BT_MPLUT_SUPPORT     = 67,
    IWL_UCODE_TLV_CAPA_MULTI_QUEUE_RX_SUPPORT   = 68,
    IWL_UCODE_TLV_CAPA_BEACON_ANT_SELECTION     = 71,
    IWL_UCODE_TLV_CAPA_BEACON_STORING       = 72,
    IWL_UCODE_TLV_CAPA_LAR_SUPPORT_V2       = 73,
    IWL_UCODE_TLV_CAPA_CT_KILL_BY_FW        = 74,
    IWL_UCODE_TLV_CAPA_TEMP_THS_REPORT_SUPPORT  = 75,
    IWL_UCODE_TLV_CAPA_CTDP_SUPPORT         = 76,
    NUM_IWL_UCODE_TLV_CAPA
};

/*
 * enumeration of ucode section.
 * This enumeration is used directly for older firmware (before 16.0).
 * For new firmware, there can be up to 4 sections (see below) but the
 * first one packaged into the firmware file is the DATA section and
 * some debugging code accesses that.
 */

typedef enum iwl_ucode_sec {
    IWL_UCODE_SECTION_DATA,
    IWL_UCODE_SECTION_INST,
}IWL_UCODE_SEC;


typedef struct iwl_ucode_capabilities {
    UINT32 Max_Probe_Length;
    UINT32 N_Scan_Channels;
    UINT32 Standard_Phy_Calibration_Size;
    UINT32 Flags;
     unsigned long Api[BITS_TO_LONGS(NUM_IWL_UCODE_TLV_API)];
    unsigned long Capa[BITS_TO_LONGS(NUM_IWL_UCODE_TLV_CAPA)];
}IWL_UCODE_CAPABILITIES;

/* one for each uCode image (inst/data, init/runtime/wowlan) */
typedef struct fw_desc {
    CONST VOID *Data;   /* vmalloc'ed data */
    UINT32 Len;     /* size in bytes */
    UINT32 offset;      /* offset in the device */
}FW_DESC;

typedef struct fw_img {
    FW_DESC Sec[IWL_UCODE_SECTION_MAX];
    BOOLEAN Is_Secure;
    BOOLEAN Is_Dual_Cpus;
}FW_IMG;


/*
 * Calibration control struct.
 * Sent as part of the phy configuration command.
 * @flow_trigger: bitmap for which calibrations to perform according to
 *		flow triggers.
 * @event_trigger: bitmap for which calibrations to perform according to
 *		event triggers.
 */

#pragma pack(1)
struct iwl_tlv_calib_ctrl {
    UINT32 Flow_Trigger;
    UINT32 Event_Trigger;
};
#pragma pack()

typedef struct iwl_tlv_calib_ctrl IWL_TLV_CALIB_CTRL;

enum iwl_fw_phy_cfg {
    FW_PHY_CFG_RADIO_TYPE_POS = 0,
    FW_PHY_CFG_RADIO_TYPE = 0x3 << FW_PHY_CFG_RADIO_TYPE_POS,
    FW_PHY_CFG_RADIO_STEP_POS = 2,
    FW_PHY_CFG_RADIO_STEP = 0x3 << FW_PHY_CFG_RADIO_STEP_POS,
    FW_PHY_CFG_RADIO_DASH_POS = 4,
    FW_PHY_CFG_RADIO_DASH = 0x3 << FW_PHY_CFG_RADIO_DASH_POS,
    FW_PHY_CFG_TX_CHAIN_POS = 16,
    FW_PHY_CFG_TX_CHAIN = 0xf << FW_PHY_CFG_TX_CHAIN_POS,
    FW_PHY_CFG_RX_CHAIN_POS = 20,
    FW_PHY_CFG_RX_CHAIN = 0xf << FW_PHY_CFG_RX_CHAIN_POS,
};

/**
 * struct iwl_fw - variables associated with the firmware
 *
 * @ucode_ver: ucode version from the ucode file
 * @fw_version: firmware version string
 * @img: ucode image like ucode_rt, ucode_init, ucode_wowlan.
 * @ucode_capa: capabilities parsed from the ucode file.
 * @enhance_sensitivity_table: device can do enhanced sensitivity.
 * @init_evtlog_ptr: event log offset for init ucode.
 * @init_evtlog_size: event log size for init ucode.
 * @init_errlog_ptr: error log offfset for init ucode.
 * @inst_evtlog_ptr: event log offset for runtime ucode.
 * @inst_evtlog_size: event log size for runtime ucode.
 * @inst_errlog_ptr: error log offfset for runtime ucode.
 * @mvm_fw: indicates this is MVM firmware
 * @cipher_scheme: optional external cipher scheme.
 */
typedef struct iwl_fw {
    UINT32 Ucode_Ver;

    CHAR8 Fw_Version[ETHTOOL_FWVERS_LEN];

    /* ucode images */
    FW_IMG Img[IWL_UCODE_TYPE_MAX];

    IWL_UCODE_CAPABILITIES Ucode_Capa;
    BOOLEAN Enhance_Sensitivity_Table;

    UINT32 Init_evtlog_Ptr, Init_evtlog_size, Init_Errlog_Ptr;
    UINT32 Inst_evtlog_Ptr, Inst_evtlog_size, Inst_Errlog_Ptr;

    IWL_TLV_CALIB_CTRL Default_Calib[IWL_UCODE_TYPE_MAX];
    UINT32 Phy_Config;
    UINT8 Valid_Tx_Ant;
    UINT8 Valid_Rx_Ant;

    BOOLEAN Mvm_Fw;


}IWL_FW;


#endif
