/**
 * @file trans.c
 *
 * @details
 * This header file contains the code to implement pci, nic and
 * memory operation for Intel UNDI driver.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2012 - 2014 Intel Corporation. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * Contact Information:
 *  Intel Linux Wireless <ilw@linux.intel.com>
 * Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 *
 * BSD LICENSE
 *
 * Copyright(c) 2012 - 2014 Intel Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Intel Corporation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include "internal.h"
#include "iwl-prph.h"
#include "IwlMvmOps.h"
#include "WirelessMemoryManager.h"
#include <Library/DebugLib.h>
#include <Protocol/PciIo.h>
#include <Uefi.h>

#define EVENT_TIME (100000)

#define PCI_LATENCY_TIMEOUT 32
#define IWL_POLL_INTERVAL 10    /* microseconds */
# define HZ 1000    /* internal timer frequency */

#define SET_BIT(bit, val)   val |= (1 << bit)
#define CLEAR_BIT(bit, val) val &= ~(1 << bit)

IWL_TRANS_CONTEXT *gIwlTransContext = NULL;

EFI_STATUS
Iwl_Trans_Pcie_Initialize(IWL_TRANS_CONTEXT *IwlTransContext)
{
    EFI_STATUS Status = EFI_SUCCESS;

    IwlTransContext->TransPcie = WifiAllocatepool(sizeof(IWL_TRANS_PCIE),__FUNCTION__);
    if(IwlTransContext->TransPcie == NULL) {
        DEBUG( (EFI_D_ERROR, "IwlPciProbe: Trans_Pcie AllocatePool error\n" ));
        Status = EFI_DEVICE_ERROR;
    }

    gIwlTransContext = IwlTransContext;
    //This function can grow based on dependencies.
    return Status;
}


void
Iwl_Trans_Pcie_Fw_Alive(
    IWL_TRANS_CONTEXT *IwlTransContext, u32 scd_addr
    )
{
    iwl_pcie_tx_start(IwlTransContext, scd_addr);
}


VOID
set_bit (
    IN UINT64 nr,
    IN unsigned long *addr
    )
{
    if(addr != NULL)
    {
        addr[nr / BITS_PER_LONG] |= (unsigned long )(1UL << (nr % BITS_PER_LONG));
    }
}
VOID
clear_bit(
    IN INTN nr,
    IN unsigned long *addr
    )
{
    if(addr != NULL)
    {
        addr[nr / BITS_PER_LONG] &= ~(1UL << (nr % BITS_PER_LONG));
    }
}
INT32
test_and_set_bit (
    IN UINT64 nr,
    IN unsigned long  *addr
    )
{
    if(((1UL << (nr % BITS_PER_LONG)) & (((unsigned long *)addr)[nr / BITS_PER_LONG])))
    {
        return 1;
    }
    else
    {
        addr[nr / BITS_PER_LONG] |= (unsigned long)(1UL << (nr % BITS_PER_LONG));
        return 0;
    }
}
INT32
test_and_clear_bit (
    IN UINT64 nr,
    IN unsigned long  *addr
    )
{
    if(((1UL << (nr % BITS_PER_LONG)) & (((unsigned long *)addr)[nr / BITS_PER_LONG])))
    {
        addr[nr / BITS_PER_LONG] &= ~(1UL << (nr % BITS_PER_LONG));
        return 1;
    }
    else
    {
        return 0;
    }
}

INT32
test_bit (
    IN UINT64 nr,
    IN const volatile void *addr
    )
{
    return ((1UL << (nr % BITS_PER_LONG)) &
            (((unsigned long *)addr)[nr / BITS_PER_LONG])) != 0;
}

VOID
Iwl_Trans_Pcie_Write32(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    IN UINT32 Ofs,
    IN UINT32 Val
    )
{
    EFI_STATUS Status;
    if(IwlTransContext != NULL)
    {
        Status = IwlTransContext->PciIoFuncs->Mem.Write(IwlTransContext->PciIoFuncs,
                                                        EfiPciIoWidthUint32,
                                                        0,
                                                        Ofs,
                                                        1,
                                                        (VOID *)(UINTN) &Val
                                                        );
        if(EFI_ERROR(Status))
        {
            DEBUG( (EFI_D_ERROR, "PcieWrite32: Pcie Mem Write error, Status = %d\n", Status ));
        }
    }
}

UINT32
Iwl_Trans_Pcie_Read32(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 Ofs
    )
{
    UINT32 Val;

    IwlTransContext->PciIoFuncs->Mem.Read(IwlTransContext->PciIoFuncs,
                                              EfiPciIoWidthUint32,
                                              0,
                                              Ofs,
                                              1,
                                              (VOID *) &Val
                                              );

    return (Val);
}

VOID
Iwl_Release_Nic_Access(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    IwlTransPcieReleaseNicAccess(IwlTransContext);
}

BOOLEAN
Iwl_Grab_Nic_Access(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    BOOLEAN Silent
    )
{
    BOOLEAN Flag;

    Flag = IwlTransPcieGrabNicAccess(IwlTransContext, false);

    return (Flag);

}


UINT32
IwlTransPcieReadPrph (
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    IN UINT32 Ofs
    )
{
    UINT32 RetVal;
    Iwl_Trans_Pcie_Write32(IwlTransContext, HBUS_TARG_PRPH_RADDR,
                           ((Ofs & 0x000FFFFF) | (3 << 24))
                           );
    RetVal =  Iwl_Trans_Pcie_Read32(IwlTransContext, HBUS_TARG_PRPH_RDAT);

    return (RetVal);
}

VOID
IwlTransPcieWritePrph (
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    IN UINT32 Ofs,
    IN UINT32 Val
    )
{
    Iwl_Trans_Pcie_Write32(IwlTransContext, HBUS_TARG_PRPH_WADDR,
                           ((Ofs & 0x000FFFFF) | (3 << 24))
                           );
    Iwl_Trans_Pcie_Write32(IwlTransContext, HBUS_TARG_PRPH_WDAT, Val);
}

UINT32
Iwl_Read32(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    IN UINT32 Ofs
    )
{
    UINT32 Value;
    Value =  Iwl_Trans_Pcie_Read32(IwlTransContext, Ofs);
    if(Value < 0)
        DEBUG( (EFI_D_INFO, "Iwl_Read32: Read Failed\n" ));
    return (Value);
}

VOID
Iwl_Write32(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 Ofs,
    UINT32 Val
    )
{
    if(IwlTransContext != NULL)
    {
        Iwl_Trans_Pcie_Write32(IwlTransContext, Ofs, Val);
    }
}


VOID
Iwl_Set_Bit(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 Reg,
    UINT32 Mask
    )
{
  if(IwlTransContext != NULL)
  {
      Iwl_Trans_Pcie_Set_Bits_Mask(IwlTransContext,Reg,Mask,Mask);
  }
}

INT32
Iwl_Poll_Bit(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 Ofs,
    UINT32 Bits,
    UINT32 Mask,
    INT32 Timeout
    )
{
    INT32 Time = 0;
    do {
        if ((Iwl_Read32(IwlTransContext, Ofs) & Mask) == (Bits & Mask))
            return (Time);
        gBS->Stall(IWL_POLL_INTERVAL);//Mandatory delay added as per linux implementation
        Time += IWL_POLL_INTERVAL;
    } while (Time < Timeout);

    return (-1);
}

u32
IwlReadDirect32 (
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    IN u32 Reg
    )
{

    u32 value = 0x5a5a5a5a;

    //unsigned long flags;
    if (Iwl_Grab_Nic_Access(IwlTransContext, FALSE)) {
        value = Iwl_Read32(IwlTransContext, Reg);
        Iwl_Release_Nic_Access(IwlTransContext);
    }
    else
        DEBUG( (EFI_D_INFO, "Iwl_Read_Direct32: Pcie Read Direct32 failure. \n" ));

    return value;
}


void
iwl_write_direct32(
    struct iwl_trans_context *IwlTransContext,
    u32 reg,
    u32 value
    )
{

    if (Iwl_Grab_Nic_Access(IwlTransContext, false)) {
        Iwl_Write32(IwlTransContext, reg, value);
        Iwl_Release_Nic_Access(IwlTransContext);
    }

}

VOID
Iwl_Trans_Write8(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 Ofs,
    UINT8 Val
    )
{
    EFI_STATUS Status;
    if(IwlTransContext != NULL)
    {
        Status = IwlTransContext->PciIoFuncs->Mem.Write(IwlTransContext->PciIoFuncs,
                                                        EfiPciIoWidthUint8,
                                                        0,
                                                        Ofs,
                                                        1,
                                                        (VOID *)(UINTN) &Val
                                                        );

        if(EFI_ERROR(Status))
            DEBUG( (EFI_D_INFO, "Iwl_Trans_Write8: Pcie Mem Write8 failure.\n" ));
    }

}

VOID
Iwl_Write8(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 ofs,
    UINT8 val
    )
{
    if(IwlTransContext != NULL)
    {
        Iwl_Trans_Write8(IwlTransContext, ofs, val);
    }
}

static
int
Iwl_Pcie_Load_Firmware_chunk(
    struct iwl_trans_context *IwlTransContext,
    u32 dst_addr,
    u64 phy_addr,
    u32 byte_cnt
    )
{
    struct iwl_trans_pcie *trans_pcie = IWL_TRANS_GET_PCIE_TRANS(IwlTransContext);
    EFI_STATUS ret;

    trans_pcie->Ucode_Write_Complete = false;

    iwl_write_direct32(IwlTransContext,
                       FH_TCSR_CHNL_TX_CONFIG_REG(FH_SRVC_CHNL),
                       FH_TCSR_TX_CONFIG_REG_VAL_DMA_CHNL_PAUSE
                       );

    iwl_write_direct32(IwlTransContext,
                       FH_SRVC_CHNL_SRAM_ADDR_REG(FH_SRVC_CHNL),
                       dst_addr
                       );

    iwl_write_direct32(IwlTransContext,
                       FH_TFDIB_CTRL0_REG(FH_SRVC_CHNL),
                       phy_addr & FH_MEM_TFDIB_DRAM_ADDR_LSB_MSK
                       );

    iwl_write_direct32(IwlTransContext,
                       FH_TFDIB_CTRL1_REG(FH_SRVC_CHNL),
                       (iwl_get_dma_hi_addr(phy_addr)
                        << FH_MEM_TFDIB_REG1_ADDR_BITSHIFT) | byte_cnt
                       );

    iwl_write_direct32(IwlTransContext,
                       FH_TCSR_CHNL_TX_BUF_STS_REG(FH_SRVC_CHNL),
                       1 << FH_TCSR_CHNL_TX_BUF_STS_REG_POS_TB_NUM |
                       1 << FH_TCSR_CHNL_TX_BUF_STS_REG_POS_TB_IDX |
                       FH_TCSR_CHNL_TX_BUF_STS_REG_VAL_TFDB_VALID
                       );

    iwl_write_direct32(IwlTransContext,
                       FH_TCSR_CHNL_TX_CONFIG_REG(FH_SRVC_CHNL),
                       FH_TCSR_TX_CONFIG_REG_VAL_DMA_CHNL_ENABLE   |
                       FH_TCSR_TX_CONFIG_REG_VAL_DMA_CREDIT_DISABLE    |
                       FH_TCSR_TX_CONFIG_REG_VAL_CIRQ_HOST_ENDTFD
                       );

    ret = gBS->Stall(10 * HZ);//Mandatory delay added as per linux implementation

    trans_pcie->Ucode_Write_Complete = true;
    return 0;
}

/**
  Unmap an address that was previously mapped using map callback.

**/


INT32
Dma_Unmap(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    IN VOID *mapping
    )
{
    EFI_STATUS Status;

    if(IwlTransContext != NULL)
    {
        Status = IwlTransContext->PciIoFuncs->Unmap(
                                                  IwlTransContext->PciIoFuncs,
                                                  mapping
                                                   );
        if (EFI_ERROR(Status))
        {
            DEBUG( (EFI_D_INFO, "Dma_Unmap: DMA Unmap failure.\n" ));
            return -1;
        }
        else
        {
            return 0;
        }
    }
    DEBUG( (EFI_D_INFO, "Dma_Unmap:Invalid input.\n" ));
    return -1;
}


INT32 Dma_Map (
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    IN VOID *CpuAddr,
    IN UINT32 NumBytes,
    IN UINT32 Direction,
    IN OUT UINT64 *DeviceAddrPtr,
    OUT VOID      **mapping
    )
{
    EFI_PHYSICAL_ADDRESS          *DevAddrPtr;
//    EFI_PCI_IO_PROTOCOL_OPERATION DirectionFlag;
    UINTN BuffSize;
    EFI_STATUS Status;

    BuffSize    = (UINTN) NumBytes;
    DevAddrPtr  = (EFI_PHYSICAL_ADDRESS *) (UINTN) DeviceAddrPtr;

    if (CpuAddr == NULL || IwlTransContext == NULL || mapping == NULL)
    {
        *DevAddrPtr = 0;
        return -1;
    }

    Status = WirelessPciMap(IwlTransContext->PciIoFuncs, Direction, CpuAddr, &BuffSize, DevAddrPtr, mapping);
    if(EFI_ERROR(Status))
    {
        DEBUG( (EFI_D_ERROR, "Dma_Map:Failed to map.\n" ));
        return -1;
    }

    return 0;
}


static
u32
iwl_trans_pcie_read_shr(
    struct iwl_trans_context *IwlTransContext,
    u32 reg
    )
{
    Iwl_Write32(IwlTransContext,
                HEEP_CTRL_WRD_PCIEX_CTRL_REG,
                ((reg & 0x0000ffff) | (2 << 28))
                );
    return Iwl_Read32(IwlTransContext, HEEP_CTRL_WRD_PCIEX_DATA_REG);
}

static
void
iwl_trans_pcie_write_shr(
    struct iwl_trans_context *IwlTransContext,
    u32 reg,
    u32 val
    )
{
    Iwl_Write32(
        IwlTransContext,
        HEEP_CTRL_WRD_PCIEX_DATA_REG,
        val
        );

    Iwl_Write32(
        IwlTransContext,
        HEEP_CTRL_WRD_PCIEX_CTRL_REG,
        ((reg & 0x0000ffff) | (3 << 28))
        );
}


VOID
IwlTransPcieReleaseNicAccess(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    struct iwl_trans_pcie *trans_pcie = NULL;

    if(IwlTransContext != NULL)
    {
        trans_pcie = IWL_TRANS_GET_PCIE_TRANS(IwlTransContext);
        if (trans_pcie->cmd_in_flight)
            goto out;

        // Clear Bit
        Iwl_Trans_Pcie_Set_Bits_Mask(IwlTransContext, CSR_GP_CNTRL,
                                  CSR_GP_CNTRL_REG_FLAG_MAC_ACCESS_REQ, 0);
    }

out:
    return;
}


BOOLEAN
IwlTransPcieGrabNicAccess (
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    BOOLEAN Silent
    )
{
    struct iwl_trans_pcie *trans_pcie = IWL_TRANS_GET_PCIE_TRANS(IwlTransContext);
    INT32 ReturnVal;
    u32 val;

    if (trans_pcie->cmd_in_flight)
        goto out;

    //This bit wakes up the NIC
    Iwl_Set_Bit(IwlTransContext, CSR_GP_CNTRL, CSR_GP_CNTRL_REG_FLAG_MAC_ACCESS_REQ);

    if (IwlTransContext->IwlConfig->Device_Family == IWL_DEVICE_FAMILY_8000)
        gBS->Stall (20);
    ReturnVal = Iwl_Poll_Bit(IwlTransContext, CSR_GP_CNTRL,
                             CSR_GP_CNTRL_REG_VAL_MAC_ACCESS_EN,
                             (CSR_GP_CNTRL_REG_FLAG_MAC_CLOCK_READY | CSR_GP_CNTRL_REG_FLAG_GOING_TO_SLEEP),
                             15000);
    if(ReturnVal > 0) {
        Iwl_Write32(IwlTransContext, CSR_RESET, CSR_RESET_REG_FLAG_FORCE_NMI);
        if(!Silent) {
            val = Iwl_Read32(IwlTransContext, CSR_GP_CNTRL);
            return false;
        }
    }

out:
    return true;
}

VOID
Iwl_Trans_Pcie_Set_Bits_Mask(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 Reg,
    UINT32 Mask,
    UINT32 Value
    )
{
    UINT32 Val;
    Val = Iwl_Read32(IwlTransContext, Reg);

    if(EFI_ERROR(Val)) {
        DEBUG( (EFI_D_INFO, "Iwl_Trans_Pcie_Set_Bits_Mask: Iwl Read32 failure.\n" ));
        return;
    }
    Val &= ~Mask;
    Val |= Value;
    Iwl_Write32(IwlTransContext, Reg, Val);

}


VOID
Iwl_Set_Bits_Prph(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 Ofs,
    UINT32 Mask
    )
{
    if(IwlTransContext != NULL)
    {
        if(Iwl_Grab_Nic_Access(IwlTransContext,false))
        {
            IwlTransPcieWritePrph(IwlTransContext, Ofs,(IwlReadPrph(IwlTransContext,Ofs) | Mask));
            Iwl_Release_Nic_Access(IwlTransContext);
        }
    }
}

VOID
Iwl_Enable_Rfkill_Int(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    struct iwl_trans_pcie *trans_pcie = NULL;

    if(IwlTransContext != NULL)
    {
        trans_pcie = IWL_TRANS_GET_PCIE_TRANS(IwlTransContext);
        trans_pcie->Inta_Mask = CSR_INT_BIT_RF_KILL;
        Iwl_Write32(IwlTransContext, CSR_INT_MASK, trans_pcie->Inta_Mask);
    }
}



int iwl_poll_bit(struct iwl_trans_context *IwlTransContext, u32 addr,
                 u32 bits, u32 mask, int timeout)
{
    int t = 0;

    do {
        if ((Iwl_Read32(IwlTransContext, addr) & mask) == (bits & mask))
            return t;
        gBS->Stall(IWL_POLL_INTERVAL*10);
        t += IWL_POLL_INTERVAL*10;
    } while (t < timeout);

    return -ETIMEDOUT;
}
#define HW_READY_TIMEOUT (50)

static int iwl_pcie_set_hw_ready(struct iwl_trans_context *IwlTransContext)
{
    int ret;

    Iwl_Set_Bit(IwlTransContext, CSR_HW_IF_CONFIG_REG, CSR_HW_IF_CONFIG_REG_BIT_NIC_READY);

    ret = iwl_poll_bit(IwlTransContext, CSR_HW_IF_CONFIG_REG,
                       CSR_HW_IF_CONFIG_REG_BIT_NIC_READY,
                       CSR_HW_IF_CONFIG_REG_BIT_NIC_READY,
                       HW_READY_TIMEOUT);

    if (ret >= 0)
        Iwl_Set_Bit(IwlTransContext, CSR_MBOX_SET_REG, CSR_MBOX_SET_REG_OS_ALIVE);

    return ret;
}
INT32
Iwl_Pcie_Prepare_Card_Hw(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    //UINT8 TimeOut = 0;
    int ret;
    int iter;
    int t = 0;

    ret = iwl_pcie_set_hw_ready(IwlTransContext);

    if (ret >= 0)
        return 0;

    Iwl_Set_Bit(IwlTransContext, CSR_DBG_LINK_PWR_MGMT_REG,
                CSR_RESET_LINK_PWR_MGMT_DISABLED);

    gBS->Stall(1000);    //1ms sleep

    for (iter = 0; iter < 10; iter++) {
        /* If HW is not ready, prepare the conditions to check again */
        Iwl_Set_Bit(IwlTransContext, CSR_HW_IF_CONFIG_REG,
                    CSR_HW_IF_CONFIG_REG_PREPARE);

        do {
            ret = iwl_pcie_set_hw_ready(IwlTransContext);
            if (ret >= 0)
                return 0;
            gBS->Stall(250);        //250 micr seconds
            t += 200;
        } while (t < 150000);
        gBS->Stall(25000);        //1ms sleep(25);
    }
//	for(TimeOut = 0; TimeOut < 5; TimeOut++){
//
//		if((Iwl_Read32(Trans, CSR_HW_IF_CONFIG_REG) & CSR_HW_IF_CONFIG_REG_BIT_NIC_READY) ==
//				CSR_HW_IF_CONFIG_REG_BIT_NIC_READY)
//			break;
//		else
//			gBS->Stall(10);  //Mandatory delay added as per linux implementation
//		if(TimeOut == 4)
//			return (-1);
//	}
    DEBUG( (EFI_D_ERROR, "Couldn't prepare the card\n" ));

    return (-1);
    //return 0;
}




INT32
Iwl_Poll_Prph_Bit(
    IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 addr,
    UINT32 bits,
    UINT32 mask,
    INT32 timeout
    )
{
    INT32 Time = 0;

    while (Time < timeout) {
        if ((IwlReadPrph(IwlTransContext, addr) & mask) == (bits & mask))
            return Time;
        gBS->Stall(IWL_POLL_INTERVAL);//Mandatory delay added as per linux implementation
        Time+=IWL_POLL_INTERVAL;
    };

    return Time;
}

void
Iwl_Clear_Bit(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    UINT32 reg,
    UINT32 mask
    )
{
    if(IwlTransContext != NULL)
    {
        Iwl_Trans_Pcie_Set_Bits_Mask(IwlTransContext, reg, mask, 0);
    }
}

VOID
Iwl_Disable_Interrupts(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    struct iwl_trans_pcie *trans_pcie = NULL;

    if(IwlTransContext != NULL)
    {
        trans_pcie = IWL_TRANS_GET_PCIE_TRANS(IwlTransContext);
        trans_pcie->Inta_Mask = (UINT32) CSR_INI_SET_MASK;
        clear_bit(STATUS_INT_ENABLED, &IwlTransContext->Status);
        Iwl_Trans_Pcie_Write32(IwlTransContext, CSR_INT_MASK, trans_pcie->Inta_Mask);
    }
}

VOID
Iwl_Enable_Interrupts(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    struct iwl_trans_pcie *trans_pcie = NULL;

    if(IwlTransContext != NULL)
    {
        trans_pcie = IWL_TRANS_GET_PCIE_TRANS(IwlTransContext);
        trans_pcie->Inta_Mask = (UINT32) CSR_INI_SET_MASK;
        set_bit(STATUS_INT_ENABLED, &IwlTransContext->Status);
        Iwl_Trans_Pcie_Write32(IwlTransContext, CSR_INT_MASK, trans_pcie->Inta_Mask);
    }
}

BOOLEAN
Iwl_Is_Rfkill_Set(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    UINT32 Status;
    Status = Iwl_Trans_Pcie_Read32(IwlTransContext, CSR_GP_CNTRL);
    Status &= CSR_GP_CNTRL_REG_FLAG_HW_RF_KILL_SW;
    DEBUG( (EFI_D_INFO, "in Iwl_Is_Rfkill_Set Status is %x\n",Status ));
    if (!Status)
        return true;
    else
        return false;
}

BOOLEAN
Iwl_Op_Mode_Hw_Rf_Kill(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    BOOLEAN state
    )
{
    //This call is implemented in the MVM ops layer
    return Iwl_Mvm_Set_Hw_Rfkill_State_Stub(state);
}


/*
 * Enable LP XTAL to avoid HW bug where device may consume much power if
 * FW is not loaded after device reset. LP XTAL is disabled by default
 * after device HW reset. Do it only if XTAL is fed by internal source.
 * Configure device's "persistence" mode to avoid resetting XTAL again when
 * SHRD_HW_RST occurs in S3.
 */
static
void
iwl_pcie_apm_lp_xtal_enable(
    struct iwl_trans_context *IwlTransContext
    )
{
    int ret;
    u32 apmg_gp1_reg;
    u32 apmg_xtal_cfg_reg;
    u32 dl_cfg_reg;


    Iwl_Trans_Pcie_Set_Bits_Mask(IwlTransContext, CSR_GP_CNTRL,
                                 CSR_GP_CNTRL_REG_FLAG_XTAL_ON,
                                 CSR_GP_CNTRL_REG_FLAG_XTAL_ON);

    /* Reset entire device - do controller reset (results in SHRD_HW_RST) */
    Iwl_Set_Bit(IwlTransContext, CSR_RESET, CSR_RESET_REG_FLAG_SW_RESET);

    gBS->Stall(10);//Mandatory delay added as per linux implementation

    /*
     * Set "initialization complete" bit to move adapter from
     * D0U* --> D0A* (powered-up active) state.
     */
    Iwl_Set_Bit(IwlTransContext, CSR_GP_CNTRL, CSR_GP_CNTRL_REG_FLAG_INIT_DONE);

    /*
     * Wait for clock stabilization; once stabilized, access to
     * device-internal resources is possible.
     */
    ret = Iwl_Poll_Bit(IwlTransContext, CSR_GP_CNTRL,
                       CSR_GP_CNTRL_REG_FLAG_MAC_CLOCK_READY,
                       CSR_GP_CNTRL_REG_FLAG_MAC_CLOCK_READY,
                       25000);
    if (ret < 0) {
        DEBUG( (EFI_D_INFO, "Access time out - failed to enable LP XTAL\n" ));
        /* Release XTAL ON request */

        Iwl_Trans_Pcie_Set_Bits_Mask(IwlTransContext, CSR_GP_CNTRL,
                                     CSR_GP_CNTRL_REG_FLAG_XTAL_ON,
                                     0);
        return;
    }

    /*
     * Clear "disable persistence" to avoid LP XTAL resetting when
     * SHRD_HW_RST is applied in S3.
     */
    IwlClearBitsPrph(IwlTransContext, APMG_PCIDEV_STT_REG,
                     APMG_PCIDEV_STT_VAL_PERSIST_DIS);

    /*
     * Force APMG XTAL to be active to prevent its disabling by HW
     * caused by APMG idle state.
     */
    apmg_xtal_cfg_reg = iwl_trans_pcie_read_shr(IwlTransContext,
                                                SHR_APMG_XTAL_CFG_REG);
    iwl_trans_pcie_write_shr(IwlTransContext, SHR_APMG_XTAL_CFG_REG,
                             apmg_xtal_cfg_reg |
                             SHR_APMG_XTAL_CFG_XTAL_ON_REQ);

    /*
     * Reset entire device again - do controller reset (results in
     * SHRD_HW_RST). Turn MAC off before proceeding.
     */
    Iwl_Set_Bit(IwlTransContext, CSR_RESET, CSR_RESET_REG_FLAG_SW_RESET);

    gBS->Stall(10);//Mandatory delay added as per linux implementation

    /* Enable LP XTAL by indirect access through CSR */
    apmg_gp1_reg = iwl_trans_pcie_read_shr(IwlTransContext, SHR_APMG_GP1_REG);
    iwl_trans_pcie_write_shr(IwlTransContext, SHR_APMG_GP1_REG, apmg_gp1_reg |
                             SHR_APMG_GP1_WF_XTAL_LP_EN |
                             SHR_APMG_GP1_CHICKEN_BIT_SELECT);

    /* Clear delay line clock power up */
    dl_cfg_reg = iwl_trans_pcie_read_shr(IwlTransContext, SHR_APMG_DL_CFG_REG);
    iwl_trans_pcie_write_shr(IwlTransContext, SHR_APMG_DL_CFG_REG, dl_cfg_reg &
                             ~SHR_APMG_DL_CFG_DL_CLOCK_POWER_UP);

    /*
     * Enable persistence mode to avoid LP XTAL resetting when
     * SHRD_HW_RST is applied in S3.
     */
    Iwl_Set_Bit(IwlTransContext, CSR_HW_IF_CONFIG_REG,
                CSR_HW_IF_CONFIG_REG_PERSIST_MODE);

    /*
     * Clear "initialization complete" bit to move adapter from
     * D0A* (powered-up Active) --> D0U* (Uninitialized) state.
     */
    Iwl_Clear_Bit(IwlTransContext, CSR_GP_CNTRL,
                  CSR_GP_CNTRL_REG_FLAG_INIT_DONE);

    /* Activates XTAL resources monitor */
    Iwl_Trans_Pcie_Set_Bits_Mask(IwlTransContext, CSR_MONITOR_CFG_REG,
                                 CSR_MONITOR_XTAL_RESOURCES, CSR_MONITOR_XTAL_RESOURCES);

    /* Release XTAL ON request */
    Iwl_Trans_Pcie_Set_Bits_Mask(IwlTransContext, CSR_GP_CNTRL,
                                 CSR_GP_CNTRL_REG_FLAG_XTAL_ON, 0);
    gBS->Stall(10);//Mandatory delay added as per linux implementation

    /* Release APMG XTAL */
    iwl_trans_pcie_write_shr(IwlTransContext, SHR_APMG_XTAL_CFG_REG,
                             apmg_xtal_cfg_reg &
                             ~SHR_APMG_XTAL_CFG_XTAL_ON_REQ);
}

static
int
iwl_pcie_apm_stop_master(
    struct iwl_trans_context *IwlTransContext
    )
{
    int ret = 0;

    /* stop device's busmaster DMA activity */
    Iwl_Set_Bit(IwlTransContext, CSR_RESET, CSR_RESET_REG_FLAG_STOP_MASTER);

    ret = Iwl_Poll_Bit(IwlTransContext, CSR_RESET,
                       CSR_RESET_REG_FLAG_MASTER_DISABLED,
                       CSR_RESET_REG_FLAG_MASTER_DISABLED, 100);
    if (ret)
        DEBUG( (EFI_D_INFO, "Master Disable Timed Out, 100 usec\n" ));

    DEBUG( (EFI_D_INFO, "stop master\n" ));

    return ret;
}

static
void
iwl_pcie_apm_stop(
    struct iwl_trans_context *IwlTransContext
    )
{
    DEBUG( (EFI_D_INFO, "Stop card, put in low power state\n" ));

    if(!IwlTransContext)
    {
        return;
    }
    clear_bit(STATUS_DEVICE_ENABLED, &IwlTransContext->Status);
    /* inform ME that we are leaving */
    if (IwlTransContext->IwlConfig->Device_Family == IWL_DEVICE_FAMILY_7000)
        Iwl_Set_Bits_Prph(IwlTransContext, APMG_PCIDEV_STT_REG,
                          APMG_PCIDEV_STT_VAL_WAKE_ME);

    if (IwlTransContext->IwlConfig->Device_Family == IWL_DEVICE_FAMILY_8000)
    {
        Iwl_Set_Bit(IwlTransContext, CSR_HW_IF_CONFIG_REG,
                    CSR_HW_IF_CONFIG_REG_PREPARE |
                    CSR_HW_IF_CONFIG_REG_ENABLE_PME);
        Iwl_Set_Bit(IwlTransContext, CSR_DBG_LINK_PWR_MGMT_REG,
                    CSR_RESET_LINK_PWR_MGMT_DISABLED);
        Iwl_Set_Bit(IwlTransContext, CSR_HW_IF_CONFIG_REG,
                    CSR_HW_IF_CONFIG_REG_PREPARE |
                    CSR_HW_IF_CONFIG_REG_ENABLE_PME);
        gBS->Stall(1000);
        Iwl_Clear_Bit(IwlTransContext, CSR_DBG_LINK_PWR_MGMT_REG,
                      CSR_RESET_LINK_PWR_MGMT_DISABLED);
    }

    /* Stop device's DMA activity */
    iwl_pcie_apm_stop_master(IwlTransContext);

    if (IwlTransContext->IwlConfig->Lp_Xtal_Workaround) {
        iwl_pcie_apm_lp_xtal_enable(IwlTransContext);
        return;
    }

    /* Reset the entire device */
    Iwl_Set_Bit(IwlTransContext, CSR_RESET, CSR_RESET_REG_FLAG_SW_RESET);

    gBS->Stall(10);//Mandatory delay added as per linux implementation
    /*
     * Clear "initialization complete" bit to move adapter from
     * D0A* (powered-up Active) --> D0U* (Uninitialized) state.
     */
    Iwl_Clear_Bit(IwlTransContext, CSR_GP_CNTRL,
                  CSR_GP_CNTRL_REG_FLAG_INIT_DONE);
}

VOID
Iwl_Trans_Pcie_Stop_Device(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    BOOLEAN hw_rfkill, was_hw_rfkill;

    if(!IwlTransContext)
    {
        return;
    }
    was_hw_rfkill = Iwl_Is_Rfkill_Set(IwlTransContext);

    /* tell the device to stop sending interrupts */
    Iwl_Disable_Interrupts(IwlTransContext);

    /*
     * If a HW restart happens during firmware loading,
     * then the firmware loading might call this function
     * and later it might be called again due to the
     * restart. So don't process again if the device is
     * already dead.
     */
    if (test_bit(STATUS_DEVICE_ENABLED, &IwlTransContext->Status)) {
        iwl_pcie_tx_stop(IwlTransContext);
        iwl_pcie_rx_stop(IwlTransContext);

        /* Power-down device's busmaster DMA clocks */
        if (IwlTransContext->IwlConfig->Device_Family != IWL_DEVICE_FAMILY_8000) {
            IwlTransPcieWritePrph(IwlTransContext,
                                   APMG_CLK_DIS_REG,
                                   APMG_CLK_VAL_DMA_CLK_RQT
                                   );
            gBS->Stall(5);//Mandatory delay added as per linux implementation
        }
    }

    /* Make sure (redundant) we've released our request to stay awake */
    Iwl_Clear_Bit(IwlTransContext, CSR_GP_CNTRL,
                  CSR_GP_CNTRL_REG_FLAG_MAC_ACCESS_REQ);

    /* Stop the device, and put it in low power state */
    iwl_pcie_apm_stop(IwlTransContext);

    /* Upon stop, the APM issues an interrupt if HW RF kill is set.
     * Clean again the interrupt here
     */
    Iwl_Disable_Interrupts(IwlTransContext);

    /* stop and reset the on-board processor */
    Iwl_Write32(IwlTransContext, CSR_RESET, CSR_RESET_REG_FLAG_NEVO_RESET);

    /* clear all status bits */
    //UEFI clear_bit needs to be implemented.
    clear_bit(STATUS_SYNC_HCMD_ACTIVE, &IwlTransContext->Status);
    clear_bit(STATUS_INT_ENABLED, &IwlTransContext->Status);
    clear_bit(STATUS_DEVICE_ENABLED, &IwlTransContext->Status);
    clear_bit(STATUS_TPOWER_PMI, &IwlTransContext->Status);
    clear_bit(STATUS_RFKILL, &IwlTransContext->Status);

    /*
     * Even if we stop the HW, we still want the RF kill
     * interrupt
     */
    Iwl_Enable_Rfkill_Int(IwlTransContext);

    /*
     * Check again since the RF kill state may have changed while
     * all the interrupts were disabled, in this case we couldn't
     * receive the RF kill interrupt and update the state in the
     * op_mode.
     * Don't call the op_mode if the rkfill state hasn't changed.
     * This allows the op_mode to call stop_device from the rfkill
     * notification without endless recursion. Under very rare
     * circumstances, we might have a small recursion if the rfkill
     * state changed exactly now while we were called from stop_device.
     * This is very unlikely but can happen and is supported.
     */
    hw_rfkill = Iwl_Is_Rfkill_Set(IwlTransContext);
    if (hw_rfkill)
        SET_BIT(STATUS_RFKILL, IwlTransContext->Status);
    else
        clear_bit(STATUS_RFKILL, &IwlTransContext->Status);
    if (hw_rfkill != was_hw_rfkill)
        Iwl_Trans_Pcie_Rf_Kill(IwlTransContext, hw_rfkill);

}

VOID
Iwl_Trans_Pcie_Rf_Kill(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    IN BOOLEAN state
    )
{

    if (Iwl_Op_Mode_Hw_Rf_Kill(IwlTransContext, state)) {
        DEBUG( (EFI_D_ERROR, "\nRF_KILL " ));
        Iwl_Trans_Pcie_Stop_Device(IwlTransContext);
    }
}

INT32
Iwl_Pcie_Nic_Init(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    INT32 Val;

    DEBUG( (EFI_D_INFO, "In PCIE NIC Init Call.\n" ));

    if(!IwlTransContext)
    {
        return -1;
    }
    set_bit(STATUS_DEVICE_ENABLED, &IwlTransContext->Status);
    /*
     * Use "set_bit" below rather than "write", to preserve any hardware
     * bits already set by default after reset.
     */

    /* Disable L0S exit timer (platform NMI Work/Around) */
    if (IwlTransContext->IwlConfig->Device_Family!= IWL_DEVICE_FAMILY_8000)
        Iwl_Set_Bit(IwlTransContext, CSR_GIO_CHICKEN_BITS,
                    CSR_GIO_CHICKEN_BITS_REG_BIT_DIS_L0S_EXIT_TIMER);

    /*
     * Disable L0s without affecting L1;
     *  don't wait for ICH L0s (ICH bug W/A)
     */
    Iwl_Set_Bit(IwlTransContext, CSR_GIO_CHICKEN_BITS,
                CSR_GIO_CHICKEN_BITS_REG_BIT_L1A_NO_L0S_RX);

    /* Set FH wait threshold to maximum (HW error during stress W/A) */
    Iwl_Set_Bit(IwlTransContext, CSR_DBG_HPET_MEM_REG, CSR_DBG_HPET_MEM_REG_VAL);

    /*
     * Enable HAP INTA (interrupt from management bus) to
     * wake device's PCI Express link L1a -> L0s
     */
    Iwl_Set_Bit(IwlTransContext, CSR_HW_IF_CONFIG_REG,
                CSR_HW_IF_CONFIG_REG_BIT_HAP_WAKE_L1A);

    /*
     * Set "initialization complete" bit to move adapter from
     * D0U* --> D0A* (powered-up active) state.
     */
    Iwl_Set_Bit(IwlTransContext, CSR_GP_CNTRL,
                CSR_GP_CNTRL_REG_FLAG_INIT_DONE);

    Val = Iwl_Poll_Bit(IwlTransContext, CSR_GP_CNTRL,
                       CSR_GP_CNTRL_REG_FLAG_MAC_CLOCK_READY,
                       CSR_GP_CNTRL_REG_FLAG_MAC_CLOCK_READY,
                       25000);

    if (Val < 0)
        goto Out;


    // Replace 1 with Host Interrupt operational mode as true.
    // Host Interrupt operational mode is part of iwl_config structure and value is true

    if (IwlTransContext->IwlConfig->Device_Id == INTEL_7260_DEVICE_ID_1 || IwlTransContext->IwlConfig->Device_Id == INTEL_7260_DEVICE_ID_2 || IwlTransContext->IwlConfig->Device_Id == INTEL_3160_DEVICE_ID) {

        /*
         * This is a bit of an abuse - This is needed for 7260 / 3160
         * only check host_interrupt_operation_mode even if this is
         * not related to host_interrupt_operation_mode.
         *
         * Enable the oscillator to count wake up time for L1 exit. This
         * consumes slightly more power (100uA) - but allows to be sure
         * that we wake up from L1 on time.
         *
         * This looks weird: read twice the same register, discard the
         * value, set a bit, and yet again, read that same register
         * just to discard the value. But that's the way the hardware
         * seems to like it.
         */

        IwlReadPrph(IwlTransContext, OSC_CLK);
        IwlReadPrph(IwlTransContext, OSC_CLK);
        Iwl_Set_Bits_Prph(IwlTransContext, OSC_CLK, OSC_CLK_FORCE_CONTROL);
        IwlReadPrph(IwlTransContext, OSC_CLK);
        IwlReadPrph(IwlTransContext, OSC_CLK);
    }

    if (IwlTransContext->IwlConfig->Device_Family != IWL_DEVICE_FAMILY_8000) {
        IwlTransPcieWritePrph(IwlTransContext, APMG_CLK_EN_REG,
                               APMG_CLK_VAL_DMA_CLK_RQT);
        gBS->Stall (20);// Mandatory delay provided for proper initialisation
        IwlTransPcieWritePrph(IwlTransContext, APMG_RTC_INT_STT_REG,
                               APMG_RTC_INT_STT_RFKILL);

        gBS->Stall (20);// Mandatory delay provided for proper initialisation

        Iwl_Set_Bits_Prph(IwlTransContext, APMG_PCIDEV_STT_REG, APMG_PCIDEV_STT_VAL_L1_ACT_DIS);

        IwlTransPcieWritePrph(IwlTransContext, APMG_RTC_INT_STT_REG,
                               APMG_RTC_INT_STT_RFKILL);
    }
    //	this function sets the PM constants so not implementing now.


    Iwl_Mvm_Nic_Config_Stub();

    Iwl_Pcie_Rx_Init(IwlTransContext);
    /* Allocate or reset and init all Tx and Command queues */
    if (iwl_pcie_tx_init(IwlTransContext))
        return -ENOMEM;


    return 0;


Out:
    return Val;
}


INT32
Iwl_Pcie_Load_Section(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    INT8 section_num,
    const struct fw_desc *Section
    )
{
    UINT64 *V_Address=NULL;
    UINT64 Cpu_Addr;
    UINT64 p_addr;
    UINT32 offset=0, chunk_sz, len;
    INT32 Ret = 0;
    EFI_STATUS Status = EFI_SUCCESS;
    VOID *mapping=NULL;

    DEBUG( (EFI_D_INFO, "Loading section %d data chunk by chunk length %d.\n", section_num,Section->Len ));

    if(Section->Len == 0)
        return (-EINVLN);

    //DMA Malloc Here

    Status =  WirelessAllocatePage((VOID **)&V_Address);
    //Status =  IwlTransContext->MemoryManager->WirelessAllocatePage((VOID **)&V_Address);

    if (EFI_ERROR(Status))
    {
        DEBUG( (EFI_D_ERROR, "Iwl_Pcie_Load_Section: No memory can be allocated for V_Address,status = %d\n",Status ));
        return -1;
    }

    Cpu_Addr = (UINT64 )V_Address;

    len = Section->Len;
    for (offset = 0; offset < Section->Len; offset += chunk_sz) {
        if (len > PAGE_SIZE)
            chunk_sz = PAGE_SIZE;
        else
            chunk_sz = len;

        CopyMem(V_Address,((CHAR8 *)Section->Data + offset),(UINTN )(chunk_sz));

        //Make DMA Map here to get Physical address
        Ret = Dma_Map(IwlTransContext, V_Address, chunk_sz, TO_DEVICE, &p_addr, &mapping);

        if (Ret < 0) {
            DEBUG( (EFI_D_ERROR, "Iwl_Pcie_Load_Section: DMA Map Failed,status = %d\n",Ret ));
            break;
        }

        Ret = Iwl_Pcie_Load_Firmware_chunk(IwlTransContext, Section->offset + offset, p_addr,chunk_sz);

        if (Ret < 0) {
            DEBUG( (EFI_D_INFO, "Ucode write failed for section number %d\n",section_num ));
            break;
        }

        //Make DMA Unmap here
        Ret = Dma_Unmap(IwlTransContext, mapping);

        if (Ret  < 0) {
            DEBUG( (EFI_D_ERROR, "Iwl_Pcie_Load_Section: DMA Unmap Failed,status = %d\n",Ret ));
            break;
        }

        len -= chunk_sz;


    }

    Status = WirelessFreePage(V_Address);

    if (EFI_ERROR(Status)) {
        DEBUG( (EFI_D_ERROR, "Iwl_Pcie_Load_Section: Memory release failed\n" ));
        Ret = -1;
    }

    return Ret;
}

INT32
Iwl_Pcie_Load_Cpu_Secured_Sections(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    const struct fw_img *fw_img,
    INT32 cpu,
    INT32 *first_ucode_section
    )
{
    INT32 Shift_Param;
    INT32 i,Ret = 0;
    UINT32 Last_Read_Index = 0;

    DEBUG( (EFI_D_INFO, "Iwl_Pcie_Load_Cpu_Secured_Sections \r\n" ));

    if(!fw_img || !first_ucode_section)
    {
        return -1;
    }

    if (cpu == 1) {
        Shift_Param = 1;
        *first_ucode_section = 0;
    }
    else {
        Shift_Param = 16;
        (*first_ucode_section)++;
    }

    for (i = *first_ucode_section; i < IWL_UCODE_SECTION_MAX; i++) {
        Last_Read_Index = i;
        if (!fw_img->Sec[i].Data ||
            fw_img->Sec[i].offset == CPU1_CPU2_SEPARATOR_SECTION) {
            break;
        }

        if (i == (*first_ucode_section) + 1)
            /* set CPU to started */
            Iwl_Set_Bits_Prph(IwlTransContext,
                              CSR_UCODE_LOAD_STATUS_ADDR,
                              LMPM_CPU_HDRS_LOADING_COMPLETED
                              << Shift_Param);

        Ret = Iwl_Pcie_Load_Section(IwlTransContext, (INT8 )i, &fw_img->Sec[i]);

        if (Ret)
            return Ret;
    }

    Iwl_Set_Bits_Prph(IwlTransContext,
                      CSR_UCODE_LOAD_STATUS_ADDR,
                      LMPM_CPU_UCODE_LOADING_COMPLETED
                      << Shift_Param);

    *first_ucode_section = Last_Read_Index;

    return 0;
}


INT32
Iwl_Pcie_Load_Cpu_Sections(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    const struct fw_img *fw_img,
    INT32 cpu,
    INT32 *first_ucode_section
    )
{
    INT32 Shift_Param;
    INT32 Ret = 0;
    UINT32 Last_Read_Index = 0;

    UINT32 i=0;

    if(!IwlTransContext || !fw_img || !first_ucode_section)
    {
        return -1;
    }
    if (cpu == 1) {
        Shift_Param = 0;
        *first_ucode_section = 0;
    } else {
        Shift_Param = 16;
        (*first_ucode_section)++;
    }

    for (i = *first_ucode_section; i < IWL_UCODE_SECTION_MAX; i++) {
        Last_Read_Index = i;


        if (fw_img->Sec[i].Len == 0) {
            break;
        }


        if ((!(fw_img->Sec[i].Data)) ||
            fw_img->Sec[i].offset == CPU1_CPU2_SEPARATOR_SECTION) {
            break;
        }

        Ret = Iwl_Pcie_Load_Section(IwlTransContext, (INT8 ) i, &fw_img->Sec[i]);

        if (Ret)
            return Ret;
    }
    if (IwlTransContext->IwlConfig->Device_Family == IWL_DEVICE_FAMILY_8000)
        Iwl_Set_Bits_Prph(IwlTransContext,
                          CSR_UCODE_LOAD_STATUS_ADDR,
                          (LMPM_CPU_UCODE_LOADING_COMPLETED |
                           LMPM_CPU_HDRS_LOADING_COMPLETED |
                           LMPM_CPU_UCODE_LOADING_STARTED) <<
                          Shift_Param);

    *first_ucode_section = Last_Read_Index;

    return 0;
}

INT32 Iwl_Pcie_Load_Cpu_Sections_8000(struct iwl_trans_context *IwlTransContext,
                                      const struct fw_img *image,
                                      int cpu,
                                      int *first_ucode_section)
{
    int Shift_Param;
    int i, ret = 0, sec_num = 0x1;
    u32 val, last_read_idx = 0;

    if(!image || !first_ucode_section)
    {
        return -1;
    }
    if (cpu == 1) {
        Shift_Param = 0;
        *first_ucode_section = 0;
    } else {
        Shift_Param = 16;
        (*first_ucode_section)++;
    }

    for (i = *first_ucode_section; i < IWL_UCODE_SECTION_MAX; i++) {
        last_read_idx = i;

        if (!image->Sec[i].Data ||
            image->Sec[i].offset == CPU1_CPU2_SEPARATOR_SECTION ||
            image->Sec[i].offset == PAGING_SEPARATOR_SECTION) {

            DEBUG( (EFI_D_INFO, "Break since Data not valid or Empty section, sec = %d\n",i ));

            break;
        }

        ret = Iwl_Pcie_Load_Section(IwlTransContext, (INT8)i, &image->Sec[i]);
        if (ret)
            return ret;

        /* Notify the ucode of the loaded section number and status */
        val = IwlReadDirect32(IwlTransContext, FH_UCODE_LOAD_STATUS);
        val = val | (sec_num << Shift_Param);
        iwl_write_direct32(IwlTransContext, FH_UCODE_LOAD_STATUS, val);
        sec_num = (sec_num << 1) | 0x1;
    }

    *first_ucode_section = last_read_idx;

    if (cpu == 1)
        iwl_write_direct32(IwlTransContext, FH_UCODE_LOAD_STATUS, 0xFFFF);
    else
        iwl_write_direct32(IwlTransContext, FH_UCODE_LOAD_STATUS, 0xFFFFFFFF);

    return 0;
}


INT32
Iwl_Pcie_Load_Given_Ucode(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    const struct fw_img *fw_img
    )
{
    INT32 Ret;
    UINT32 IntRegVal = 0;
    INT32 first_ucode_section;

    if(!IwlTransContext || !fw_img)
    {
        return -1;
    }

    if (fw_img->Is_Secure) {
        /* set secure boot inspector addresses */
        IwlTransPcieWritePrph(IwlTransContext, LMPM_SECURE_INSPECTOR_CODE_ADDR,
                               LMPM_SECURE_INSPECTOR_CODE_MEM_SPACE);

        IwlTransPcieWritePrph(IwlTransContext, LMPM_SECURE_INSPECTOR_DATA_ADDR,
                               LMPM_SECURE_INSPECTOR_DATA_MEM_SPACE);

        /* set CPU1 header address */
        IwlTransPcieWritePrph(IwlTransContext, LMPM_SECURE_UCODE_LOAD_CPU1_HDR_ADDR,
                               LMPM_SECURE_CPU1_HDR_MEM_SPACE);

        /* load to FW the binary Secured sections of CPU1 */
        Ret = Iwl_Pcie_Load_Cpu_Secured_Sections(IwlTransContext, fw_img, 1,
                                                 &first_ucode_section);
        if (Ret)
            return Ret;
    }
    else {

        /* load to FW the binary Non secured sections of CPU1 */
        Ret = Iwl_Pcie_Load_Cpu_Sections(IwlTransContext, fw_img, 1,
                                         &first_ucode_section);
        if (Ret)
            return Ret;
    }

    if (fw_img->Is_Dual_Cpus) {
        IwlTransPcieWritePrph(IwlTransContext, LMPM_SECURE_UCODE_LOAD_CPU2_HDR_ADDR,
                               LMPM_SECURE_CPU2_HDR_MEM_SPACE);


        if (fw_img->Is_Secure) {
            Ret = Iwl_Pcie_Load_Cpu_Secured_Sections(IwlTransContext, fw_img, 2,
                                                     &first_ucode_section);
        }
        else {
            Ret = Iwl_Pcie_Load_Cpu_Sections(IwlTransContext, fw_img, 2,
                                             &first_ucode_section);
        }

        if (Ret)
            return Ret;
    }

    DEBUG( (EFI_D_INFO, "Reset NIC.\n" ));
    IntRegVal = Iwl_Read32(IwlTransContext, CSR_INT);

    DEBUG( (EFI_D_INFO, "Before NIC Reset: INT Reg Status = %x\n",IntRegVal ));

    /* release CPU reset */
    Iwl_Write32(IwlTransContext, CSR_RESET, 0);

    DEBUG( (EFI_D_INFO, "NIC Reset Done.\n" ));

    gBS->Stall(100);//// Mandatory delay  required after nic reset

    IntRegVal = Iwl_Read32(IwlTransContext, CSR_INT);
    DEBUG( (EFI_D_INFO, "After NIC Reset: INT Reg Status = %x\n",IntRegVal ));

    if(IntRegVal & CSR_INT_BIT_ALIVE) {


        DEBUG( (EFI_D_INFO, "NIC ALIVE Bit is set in CSR_INT Register.\n" ));
        DEBUG( (EFI_D_INFO, "Ready to start MVM commands to UCODE.\n" ));
    }

    if (fw_img->Is_Secure) {
        /* wait for image verification to complete  */
        Ret = Iwl_Poll_Prph_Bit(IwlTransContext,
                                LMPM_SECURE_BOOT_CPU1_STATUS_ADDR,
                                LMPM_SECURE_BOOT_STATUS_SUCCESS,
                                LMPM_SECURE_BOOT_STATUS_SUCCESS,
                                LMPM_SECURE_TIME_OUT);

        if (Ret < 0) {
            /* Time out on secure boot process*/
            return Ret;
        }
    }
    return 0;
}
INT32 Iwl_Pcie_Load_Given_Ucode_8000(struct iwl_trans_context *IwlTransContext,
                                     const struct fw_img *image)
{
    int ret = 0;
    int first_ucode_section;



    /* configure the ucode to be ready to get the secured image */
    /* release CPU reset */
    IwlWritePrph(IwlTransContext, RELEASE_CPU_RESET, RELEASE_CPU_RESET_BIT);
    if (ret)
        return ret;

    /* load to FW the binary Secured sections of CPU1 */
    ret = Iwl_Pcie_Load_Cpu_Sections_8000(IwlTransContext, image, 1,
                                          &first_ucode_section);
    if (ret)
        return ret;

    /* load to FW the binary sections of CPU2 */
    ret = Iwl_Pcie_Load_Cpu_Sections_8000(IwlTransContext, image, 2,
                                          &first_ucode_section);


    return 0;
}


/*
 *  Start the HW here on
 */

INT32
Iwl_Trans_Pcie_Start_Hw(
    IN IWL_TRANS_CONTEXT *IwlTransContext
    )
{
    UINT32 TimeOut = 0;
    BOOLEAN hw_rfkill;

    if(!IwlTransContext)
    {
        return -1;
    }

    if ((Iwl_Pcie_Prepare_Card_Hw(IwlTransContext)) < 0)
        return -1;
    /**
     * Reset the Entire Device.
     **/

    Iwl_Write32(IwlTransContext,CSR_RESET,CSR_RESET_REG_FLAG_SW_RESET);

    gBS->Stall(15);//Mandatory delay added as per linux implementation

    /*
     * Start up NIC's basic functionality after it has been reset
     * (e.g. after platform boot, or shutdown via iwl_pcie_apm_stop())
     * NOTE:  This does not load uCode nor start the embedded processor
     */

    /* Disable L0S exit timer (platform NMI Work/Around) */
    if (IwlTransContext->IwlConfig->Device_Family != IWL_DEVICE_FAMILY_8000)
        Iwl_Set_Bit(IwlTransContext, CSR_GIO_CHICKEN_BITS,
                    CSR_GIO_CHICKEN_BITS_REG_BIT_DIS_L0S_EXIT_TIMER);

    /*
     * Disable L0s without affecting L1;
     *  don't wait for ICH L0s (ICH bug W/A)
     */
    Iwl_Set_Bit(IwlTransContext, CSR_GIO_CHICKEN_BITS,
                CSR_GIO_CHICKEN_BITS_REG_BIT_L1A_NO_L0S_RX);

    /* Set FH wait threshold to maximum (HW error during stress W/A) */
    Iwl_Set_Bit(IwlTransContext, CSR_DBG_HPET_MEM_REG, CSR_DBG_HPET_MEM_REG_VAL);

    /*
     * Enable HAP INTA (interrupt from management bus) to
     * wake device's PCI Express link L1a -> L0s
     */
    Iwl_Set_Bit(IwlTransContext, CSR_HW_IF_CONFIG_REG,
                CSR_HW_IF_CONFIG_REG_BIT_HAP_WAKE_L1A);


    //
    // Next API Needs to be implemented if required.
    // This deals with PM settings.
    //



    /*
     * Set "initialization complete" bit to move adapter from
     * D0U* --> D0A* (powered-up active) state.
     */
    Iwl_Set_Bit(IwlTransContext, CSR_GP_CNTRL, CSR_GP_CNTRL_REG_FLAG_INIT_DONE);


    /*
     * Wait for clock stabilization; once stabilized, access to
     * device-internal resources is supported, e.g. iwl_write_prph()
     * and accesses to uCode SRAM.
     */

    for(TimeOut = 0; TimeOut <= 25000; (TimeOut = TimeOut+10))
    {
        if((Iwl_Read32(IwlTransContext, CSR_GP_CNTRL) & CSR_GP_CNTRL_REG_FLAG_MAC_CLOCK_READY) ==
           CSR_GP_CNTRL_REG_FLAG_MAC_CLOCK_READY)
            break;
        else
            gBS->Stall(15);//Mandatory delay  provided  for proper initialisation
        if(TimeOut >= 25000)
            return (-1);

    }

    // Replace 1 with Host Interrupt operational mode as true.
    // Host Interrupt operational mode is part of iwl_config structure and value is true

    if (IwlTransContext->IwlConfig->Device_Id == INTEL_7260_DEVICE_ID_1 || IwlTransContext->IwlConfig->Device_Id == INTEL_7260_DEVICE_ID_2 ||(IwlTransContext->IwlConfig->Device_Id == INTEL_3160_DEVICE_ID)) {

        /*
         * This is a bit of an abuse - This is needed for 7260 / 3160
         * only check host_interrupt_operation_mode even if this is
         * not related to host_interrupt_operation_mode.
         *
         * Enable the oscillator to count wake up time for L1 exit. This
         * consumes slightly more power (100uA) - but allows to be sure
         * that we wake up from L1 on time.
         *
         * This looks weird: read twice the same register, discard the
         * value, set a bit, and yet again, read that same register
         * just to discard the value. But that's the way the hardware
         * seems to like it.
         */

        IwlReadPrph(IwlTransContext, OSC_CLK);
        IwlReadPrph(IwlTransContext, OSC_CLK);
        Iwl_Set_Bits_Prph(IwlTransContext, OSC_CLK, OSC_CLK_FORCE_CONTROL);
        IwlReadPrph(IwlTransContext, OSC_CLK);
        IwlReadPrph(IwlTransContext, OSC_CLK);

    }

    /*
     * Enable DMA clock and wait for it to stabilize.
     *
     * Write to "CLK_EN_REG"; "1" bits enable clocks, while "0"
     * bits do not disable clocks.  This preserves any hardware
     * bits already set by default in "CLK_CTRL_REG" after reset.
     */
    if (IwlTransContext->IwlConfig->Device_Family != IWL_DEVICE_FAMILY_8000) {

        IwlTransPcieWritePrph(IwlTransContext, APMG_CLK_EN_REG,
                               APMG_CLK_VAL_DMA_CLK_RQT);
        gBS->Stall(20);//Mandatory delay provided for proper initialisation

        /* Disable L1-Active */
        Iwl_Set_Bits_Prph(IwlTransContext, APMG_PCIDEV_STT_REG,
                          APMG_PCIDEV_STT_VAL_L1_ACT_DIS);

        /* Clear the interrupt in APMG if the NIC is in RFKILL */

        IwlTransPcieWritePrph(IwlTransContext, APMG_RTC_INT_STT_REG,
                               APMG_RTC_INT_STT_RFKILL);

    }

    Iwl_Enable_Rfkill_Int(IwlTransContext);

    hw_rfkill = Iwl_Is_Rfkill_Set(IwlTransContext);
    if (hw_rfkill)
        set_bit(STATUS_RFKILL, &IwlTransContext->Status);
    else
        clear_bit(STATUS_RFKILL, &IwlTransContext->Status);
    //	The Below Function is used for the recovery mechanism
    //	in case of NIC crash / Reset.
    //	This Funciton will check the driver status and either
    //	continue or stop the device.
    Iwl_Trans_Pcie_Rf_Kill(IwlTransContext, hw_rfkill);

    return (EFI_SUCCESS);
}


/*
 *  Start Loading FW here on
 */

INT32
Iwl_Trans_Pcie_Start_Fw(
    IN IWL_TRANS_CONTEXT *IwlTransContext,
    const struct fw_img *fw,
    BOOLEAN run_in_rfkill
    )
{
    INT32 Ret;
    BOOLEAN Hw_RfKill;

    DEBUG( (EFI_D_INFO, "Iwl_Trans_Pcie_Start_Fw length is %d\r\n",fw->Sec->Len ));

    if(!IwlTransContext)
    {
        return -1;
    }
    if (Iwl_Pcie_Prepare_Card_Hw(IwlTransContext) < 0)
        return -1;

    //Enable RF Kill Int
    Iwl_Enable_Rfkill_Int(IwlTransContext);

    Hw_RfKill = Iwl_Is_Rfkill_Set(IwlTransContext);
    if (Hw_RfKill)
        set_bit(STATUS_RFKILL, &IwlTransContext->Status);
    else
        clear_bit(STATUS_RFKILL, &IwlTransContext->Status);
//	Below function is used to stop device in case if NIC is not ready to flash.
    DEBUG( (EFI_D_INFO, "in Iwl_Trans_Pcie_Start_Fw Hw_RfKill is %d %d\n",Hw_RfKill,run_in_rfkill ));
    Iwl_Trans_Pcie_Rf_Kill(IwlTransContext, Hw_RfKill);
    if (Hw_RfKill && !run_in_rfkill)
        return -ERFKILL;

    Iwl_Write32(IwlTransContext,CSR_INT_MASK,0xFFFFFFFF);

    Ret = Iwl_Pcie_Nic_Init(IwlTransContext);
    if (Ret < 0)
        return Ret;
    /* make sure rfkill handshake bits are cleared */

    Iwl_Write32(IwlTransContext,CSR_UCODE_DRV_GP1_CLR,CSR_UCODE_SW_BIT_RFKILL);
    Iwl_Write32(IwlTransContext,CSR_UCODE_DRV_GP1_CLR,CSR_UCODE_DRV_GP1_BIT_CMD_BLOCKED);

    /*Clear all Interrupts and enable host ints*/
    Iwl_Write32(IwlTransContext,CSR_INT_MASK,0xFFFFFFFF);
    Iwl_Write32(IwlTransContext,CSR_INT_MASK,(UINT32)CSR_INI_SET_MASK);

    /* really make sure rfkill handshake bits are cleared */
    Iwl_Write32(IwlTransContext,CSR_UCODE_DRV_GP1_CLR,CSR_UCODE_SW_BIT_RFKILL);
    Iwl_Write32(IwlTransContext,CSR_UCODE_DRV_GP1_CLR,CSR_UCODE_SW_BIT_RFKILL);
    if (IwlTransContext->IwlConfig->Device_Family == IWL_DEVICE_FAMILY_8000)
        Ret= Iwl_Pcie_Load_Given_Ucode_8000(IwlTransContext, fw);
    else
        Ret = Iwl_Pcie_Load_Given_Ucode (IwlTransContext, fw);

    return Ret;
}
INTN
Iwl_Trans_Pcie_Read_Mem(
    struct iwl_trans_context *IwlTransContext,
    u32 addr,
    void *buf,
    INTN dwords
    )
{

    INTN offs, ret = 0;
    UINT32 *vals = buf;

    if (IwlTransPcieGrabNicAccess(IwlTransContext,false)) {
        Iwl_Write32(IwlTransContext, HBUS_TARG_MEM_RADDR, addr);
        for (offs = 0; offs < dwords; offs++)
            vals[offs] = Iwl_Read32(IwlTransContext, HBUS_TARG_MEM_RDAT);
        IwlTransPcieReleaseNicAccess(IwlTransContext);
    } else {
        ret = -EBUSY;
    }
    return ret;
}

void
iwl_stop_queue(
    struct iwl_trans_context *IwlTransContext,
    struct iwl_txq *txq
    )
{
    IWL_TRANS_PCIE *trans_pcie;

    if(IwlTransContext != NULL && txq != NULL)
    {
        trans_pcie = IWL_TRANS_GET_PCIE_TRANS(IwlTransContext);

        if (!test_and_set_bit(txq->q.id, trans_pcie->queue_stopped)) {

            iwl_op_mode_cmd_queue_full_Stub((int)(txq->q.id));

            DEBUG( (EFI_D_ERROR, "IwlTransContext--Stop hwq %d\n", txq->q.id ));
    }
    else
            DEBUG( (EFI_D_ERROR,"IwlTransContext--hwq %d already stopped\n",
                                                   txq->q.id));
    }
}

INTN
Iwl_Trans_Pcie_Write_Mem(
    struct iwl_trans_context *IwlTransContext,
    u32 addr,
    const void *buf,
    INTN dwords
    )
{
    INTN offs;
    INTN ret = 0;
    const u32 *vals = buf;

    if (IwlTransPcieGrabNicAccess(IwlTransContext,false)) {
        Iwl_Write32(IwlTransContext, HBUS_TARG_MEM_WADDR, addr);
        for (offs = 0; offs < dwords; offs++)
            Iwl_Write32(IwlTransContext, HBUS_TARG_MEM_WDAT,
                        vals ? vals[offs] : 0);
        IwlTransPcieReleaseNicAccess(IwlTransContext);
    } else {
        ret = -EBUSY;
    }
    return ret;
}

void
Iwl_Trans_Pcie_Configure(
    struct iwl_trans_context *IwlTransContext,
    const struct iwl_trans_config *trans_cfg
    )
{

    struct iwl_trans_pcie *trans_pcie = NULL;

    DEBUG( (EFI_D_INFO, "In Trans pcie configure.\n" ));

    if(IwlTransContext != NULL && trans_cfg != NULL)
    {
        trans_pcie = IWL_TRANS_GET_PCIE_TRANS(IwlTransContext);
        trans_pcie->Cmd_Queue = trans_cfg->cmd_queue;
        trans_pcie->cmd_fifo = trans_cfg->cmd_fifo;
        if (WARN_ON(trans_cfg->n_no_reclaim_cmds > MAX_NO_RECLAIM_CMDS))
            trans_pcie->n_no_reclaim_cmds = 0;
        else
            trans_pcie->n_no_reclaim_cmds = (UINT8)trans_cfg->n_no_reclaim_cmds;
        if (trans_pcie->n_no_reclaim_cmds)
            CopyMem(trans_pcie->no_reclaim_cmds, trans_cfg->no_reclaim_cmds,
                    trans_pcie->n_no_reclaim_cmds * sizeof(u8));

        trans_pcie->rx_page_order = 0;

        trans_pcie->bc_table_dword = trans_cfg->bc_table_dword;
    }
}


void
Iwl_Trans_Pcie_Free(
    struct iwl_trans_context *IwlTransContext
    )
{
    DEBUG( (EFI_D_INFO, "In Iwl_Trans_Pcie_Free.\n" ));
    iwl_pcie_tx_free(IwlTransContext);
    iwl_pcie_rx_free(IwlTransContext);
}

EFI_STATUS WirelessPciMap(
    IN EFI_PCI_IO_PROTOCOL            *PciInstance,
    IN UINT32                          Direction,
    IN VOID                           *HostAddress,
    IN OUT UINTN                      *NumberOfBytes,
    OUT EFI_PHYSICAL_ADDRESS          *DeviceAddress,
    OUT VOID                          **Mapping)
{

    EFI_PCI_IO_PROTOCOL_OPERATION      Operation;
    UINTN                              BytesTobeMapped;
    EFI_STATUS                         Status;

    if (NULL == PciInstance || NULL == HostAddress || NULL == NumberOfBytes ||
        NULL == DeviceAddress || NULL == Mapping)
    {
        return EFI_INVALID_PARAMETER;
    }

    switch (Direction)
    {
        case TO_AND_FROM_DEVICE:
            Operation = EfiPciIoOperationBusMasterCommonBuffer;
            break;

        case FROM_DEVICE:
            Operation = EfiPciIoOperationBusMasterWrite;
            break;

        case TO_DEVICE:
            Operation = EfiPciIoOperationBusMasterRead;
            break;

        default:
            *Mapping = NULL;
            return EFI_INVALID_PARAMETER;
    }

    BytesTobeMapped = *NumberOfBytes;

    Status = PciInstance->Map(PciInstance, Operation, HostAddress, NumberOfBytes, DeviceAddress, Mapping);

    if (Status == EFI_SUCCESS)
    {
        //Some times actual bytes mapped may not be equal to bytes requested. Hence checking
        if (BytesTobeMapped != *NumberOfBytes)
        {
            WirelessPciUnMap(PciInstance, *Mapping);
            return EFI_NO_MAPPING;
        }
    }

    return Status;
}


EFI_STATUS WirelessPciUnMap(IN EFI_PCI_IO_PROTOCOL *PciInstance, IN VOID *Mapping)
{
    if (NULL == PciInstance || NULL == Mapping)
    {
        return EFI_INVALID_PARAMETER;
    }
    return(PciInstance->Unmap(PciInstance, Mapping));
}
