/**
 * @file IwlTransMain.h
 *
 * @details
 * This header file contains protocol definition for the
 * Intel Transport driver.
 *
 * @copyright
 * Copyright (c) 2017 Dell Inc. All rights reserved. This
 * software and associated documentation (if any) is furnished
 * under a license and may only be used or copied in accordance
 * with the terms of the license. Except as permitted by such
 * license, no part of this software or documentation may be
 * reproduced, stored in a retrieval system, or transmitted in
 * any form or by any means without the express written consent
 * of Dell Inc.
 *
 * Dell elects to use only the BSD license for any software where
 * a choice of BSD/GPLv2 license versions are made available with
 * the language indicating that BSD/GPLv2 or any later version may
 * be used, or where a choice of which version of the BSD/GPLv2 is
 * applied is unspecified.
 */

/******************************************************************************
 *
 * This file is provided under a dual BSD/GPLv2 license.  When using or
 * redistributing this file, you may do so under either license.
 *
 * GPL LICENSE SUMMARY
 *
 * Copyright(c) 2017 Dell Inc. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110,
 * USA
 *
 * The full GNU General Public License is included in this distribution
 * in the file called COPYING.
 *
 * BSD LICENSE
 *
 * Copyright(c) 2007 Dell Corporation. All rights reserved.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name Dell Inc nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef IWLTRANSMAIN_H_
#define IWLTRANSMAIN_H_

#include "iwl-trans-context.h"
#include "mvm-context.h"

typedef EFI_STATUS (*IWL_TRANS_OPS_INITIALIZE)(IWL_TRANS_CONTEXT *iwlTransPcieCtxt);

typedef VOID (*IWL_TRANS_OPS_FREE)(IWL_TRANS_CONTEXT *iwlTransContext);

typedef INT32 (*IWL_TRANS_OPS_START_HW)(IN IWL_TRANS_CONTEXT *IwlTransContext);

typedef INT32 (*IWL_TRANS_OPS_START_FW)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                        IN const struct fw_img *fw,
                                        BOOLEAN run_in_rfkill);

typedef VOID (*IWL_TRANS_OPS_FW_ALIVE)( IN struct iwl_trans_context *IwlTransContext,
                                        UINT32 scd_addr);

typedef INTN (*IWL_TRANS_OPS_SEND_CMD)( IN struct iwl_trans_context *IwlTransContext,
                                        IN struct Iwl_Host_Cmd *cmd);

typedef INTN (*IWL_TRANS_OPS_TX)(IN struct iwl_trans_context *IwlTransContext,
                                 IN SK_BUFF *skb,
                                 IN struct iwl_device_cmd *dev_cmd,
                                 INTN queue);

typedef VOID (*IWL_TRANS_OPS_RECLAIM)(struct iwl_trans_context *IwlTransContext,
                                      int queue,
                                      int ssn,
                                      struct iwl_tx_skb *skb);

typedef VOID (*IWL_TRANS_OPS_STOP_DEVICE)( IN struct iwl_trans_context *IwlTransContext);

typedef VOID (*IWL_TRANS_OPS_WRITE8)(IN struct iwl_trans_context *IwlTransContext,
                                     IN UINT32 ofs,
                                     IN UINT8 val);

typedef VOID (*IWL_TRANS_OPS_TXQ_ENABLE)(IN struct iwl_trans_context *IwlTransContext,
                                         IN UINT32 queue,
                                         IN UINT32 fifo,
                                         IN INTN sta_id,
                                         IN UINT32 tid,
                                         IN UINT32 frame_limit,
                                         IN UINT16 ssn);

typedef VOID (*IWL_TRANS_OPS_TXQ_DISABLE)(IN struct iwl_trans_context *IwlTransContext,
                                          IN int txq_id);

typedef VOID (*IWL_TRANS_OPS_WRITE32)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                      IN UINT32 Ofs,
                                      IN UINT32 Val);

typedef UINT32 (*IWL_TRANS_OPS_READ32)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                       UINT32 Ofs);

typedef UINT32 (*IWL_TRANS_OPS_READ_PRPH)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                        UINT32 Ofs);

typedef VOID (*IWL_TRANS_OPS_WRITE_PRPH)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                         IN UINT32 ofs,
                                         IN UINT32 val);

typedef INTN (*IWL_TRANS_OPS_READ_MEM)(IN struct iwl_trans_context *IwlTransContext,
                                       IN UINT32 addr,
                                       OUT VOID *buf,
                                       INTN dwords);

typedef INTN (*IWL_TRANS_OPS_WRITE_MEM)(IN struct iwl_trans_context *IwlTransContext,
                                        IN UINT32 addr,
                                        const VOID *buf,
                                        INTN dwords);

typedef VOID (*IWL_TRANS_OPS_CONFIGURE)(IN struct iwl_trans_context *IwlTransContext,
                                        IN const struct iwl_trans_config *trans_cfg);

typedef BOOLEAN (*IWL_TRANS_OPS_GRAB_NIC_ACCESS)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                              BOOLEAN Silent);

typedef VOID (*IWL_TRANS_OPS_RELEASE_NIC_ACCESS)(IN IWL_TRANS_CONTEXT *IwlTransContext);
typedef VOID (*IWL_TRANS_OPS_SET_BITS_MASK)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                            UINT32 Reg,
                                            UINT32 Mask,
                                            UINT32 Value);

typedef INT32 (*IWL_TRANS_OPS_DMA_MAP)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                       IN VOID *CpuAddr,
                                       IN UINT32 NumBytes,
                                       IN UINT32 Direction,
                                       IN OUT UINT64 *DeviceAddrPtr,
                                       OUT VOID      **mapping);

typedef INT32 (*IWL_TRANS_OPS_DMA_UNMAP)(IN IWL_TRANS_CONTEXT *IwlTransContext,
                                       IN VOID *mapping);

typedef VOID (*IWL_TRANS_OPS_IRQ_HANDLER)(IN IWL_TRANS_CONTEXT *IwlTransContext);

typedef VOID (*IWL_MVM_RS_RATE_INIT)(IN struct rate_scale_context *RateScaleContext,
                                    WIFI_STATION *sta,
                                    enum WIFI_BAND band,
                                    bool init
                                    );

typedef VOID* (*IWL_MVM_RS_ALLOC_STA)(WIFI_STATION *sta);

typedef VOID (*IWL_MVM_RS_TX_STATUS)(WIFI_SUPPORTING_BAND *sband,
                                    WIFI_STATION *sta, void *priv_sta,
                                    struct iwl_tx_skb *skb
                                    );

typedef VOID (*IWL_TRANS_OPS_REF)(IN IWL_TRANS_CONTEXT *IwlTransContext);

typedef VOID (*IWL_TRANS_OPS_UNREF)(IN IWL_TRANS_CONTEXT *IwlTransContext);

typedef VOID (*GET_FIRMWARE_INFO)(char *name);

typedef unsigned long (*GET_FW_UCODE_HEX_LENGTH)();

typedef VOID (*GET_FW_UCODE_HEX)(CHAR8* ucodehex);

typedef struct _INTEL_TRANS_OPS_PROTOCOL {
    IWL_TRANS_OPS_INITIALIZE  TransInitialize;
    IWL_TRANS_OPS_FREE        Free;
    IWL_TRANS_OPS_START_HW    StartHw;
    IWL_TRANS_OPS_START_FW    StartFw;
    IWL_TRANS_OPS_FW_ALIVE    FwAlive;
    IWL_TRANS_OPS_SEND_CMD    SendCmd;
    IWL_TRANS_OPS_TX          Tx;
    IWL_TRANS_OPS_RECLAIM     Reclaim;
    IWL_TRANS_OPS_STOP_DEVICE StopDevice;
    IWL_TRANS_OPS_WRITE8      Write8;
    IWL_TRANS_OPS_TXQ_ENABLE  TxqEnable;
    IWL_TRANS_OPS_TXQ_DISABLE TxqDisable;
    IWL_TRANS_OPS_WRITE32     Write32;
    IWL_TRANS_OPS_READ32      Read32;
    IWL_TRANS_OPS_READ_PRPH   ReadPrph;
    IWL_TRANS_OPS_WRITE_PRPH  WritePrph;
    IWL_TRANS_OPS_READ_MEM    ReadMem;
    IWL_TRANS_OPS_WRITE_MEM   WriteMem;
    IWL_TRANS_OPS_CONFIGURE   Configure;
    IWL_TRANS_OPS_GRAB_NIC_ACCESS GrabNicAccess;
    IWL_TRANS_OPS_RELEASE_NIC_ACCESS ReleaseNicAccess;
    IWL_TRANS_OPS_SET_BITS_MASK   SetBitsMask;
    IWL_TRANS_OPS_DMA_MAP     DmaMap;
    IWL_TRANS_OPS_DMA_UNMAP   DmaUnmap;
    IWL_TRANS_OPS_IRQ_HANDLER IrqHandler;
    IWL_MVM_RS_RATE_INIT      RsInit;
    IWL_MVM_RS_ALLOC_STA      RsAllocaSta;
    IWL_MVM_RS_TX_STATUS      RsTxStatus;
    IWL_TRANS_OPS_REF         Ref;
    IWL_TRANS_OPS_UNREF       Unref;

}INTEL_TRANS_OPS_PROTOCOL;

typedef struct _INTEL_FW_PROTOCOL {
    GET_FIRMWARE_INFO         GetFirmwareInfo;
    GET_FW_UCODE_HEX_LENGTH   GetFWUcodeHexLen;
    GET_FW_UCODE_HEX          GetFWUcodeHex;
}INTEL_FW_PROTOCOL;

#endif 
